--
-- PostgreSQL database dump
--

SET SESSION AUTHORIZATION 'postgres';

SET search_path = public, pg_catalog;

--
-- TOC entry 2 (OID 106034)
-- Name: kaspa_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE kaspa_seq
    START 1
    INCREMENT 1
    MAXVALUE 2147483647
    MINVALUE 1
    CACHE 1;


CREATE TABLE author (
    "no" integer DEFAULT nextval('kaspa_seq'::text),
    lastname text DEFAULT '',
    firstname text DEFAULT '',
    pseudonym text DEFAULT '',
    born text DEFAULT '',
    died text DEFAULT '',
    country text DEFAULT '',
    memo text DEFAULT '',
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    primary key(no)
);


--
-- TOC entry 6 (OID 106051)
-- Name: part; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE part (
    "no" integer DEFAULT nextval('kaspa_seq'::text),
    publication_no integer DEFAULT 0,
    partno smallint DEFAULT 1,
    title text DEFAULT '',
    pages text DEFAULT '',
    memo text DEFAULT '',
    "key" text DEFAULT '',
    "language" text DEFAULT '',
    bibtex text DEFAULT '',
    intro_no integer DEFAULT 0,
    createbibtex boolean DEFAULT false,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    primary key(no)
);


--
-- TOC entry 7 (OID 106069)
-- Name: partdata; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE partdata (
    "no" integer DEFAULT nextval('kaspa_seq'::text),
    part_no integer DEFAULT 0,
    file text DEFAULT '',
    astext text DEFAULT '',
    filename text DEFAULT '',
    "type" text DEFAULT '',
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    primary key(no)
);


--
-- TOC entry 8 (OID 106082)
-- Name: publisher; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE publisher (
    "no" integer DEFAULT nextval('kaspa_seq'::text),
    name text DEFAULT '',
    city text DEFAULT '',
    serie text DEFAULT '',
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    primary key(no)
);

CREATE TABLE journal (
    "no" integer DEFAULT nextval('kaspa_seq'::text),
    name text DEFAULT '',
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    primary key(no)
);


--
-- TOC entry 9 (OID 106093)
-- Name: publication; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE publication (
    "no" integer DEFAULT nextval('kaspa_seq'::text),
    title text DEFAULT '',
    subtitle text DEFAULT '',
    edition smallint DEFAULT 0,
    editor text DEFAULT '',
    howpublished text DEFAULT '',
    organization text DEFAULT '',
    publisher_no integer DEFAULT 0,
    journal_no integer DEFAULT 0,
    "year" text DEFAULT '',
    pages text DEFAULT '',
    translator text DEFAULT '',
    volume text DEFAULT '',
    number text DEFAULT '',
    "month" text DEFAULT '',
    series text DEFAULT '',
    "type" text DEFAULT '',
    "key" text DEFAULT '',
    orgtitle text DEFAULT '',
    orgpublisher_no integer DEFAULT 0,
    orgyear text DEFAULT '',
    isbn_issn text DEFAULT '',
    hidingplace text DEFAULT '',
    bibtex text DEFAULT '',
    entrytype smallint DEFAULT 0,
    memo text DEFAULT 0,
    createbibtex boolean DEFAULT false,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    primary key(no)
);


--
-- TOC entry 10 (OID 106126)
-- Name: publication_author; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE publication_author (
    publication_no integer DEFAULT 0,
    author_no integer DEFAULT 0,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    author_order integer DEFAULT 0
);


--
-- TOC entry 11 (OID 106133)
-- Name: part_author; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE part_author (
    part_no integer DEFAULT 0,
    author_no integer DEFAULT 0,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    author_order integer DEFAULT 0
);


--
-- TOC entry 12 (OID 106140)
-- Name: note; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE note (
    "no" integer DEFAULT nextval('kaspa_seq'::text),
    title text DEFAULT '',
    memo text DEFAULT 0,
    "type" smallint DEFAULT 0,
    publication_no integer DEFAULT 0,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    parent integer DEFAULT 0,
    sibling integer DEFAULT 0,
    primary key(no)
);


--
-- TOC entry 13 (OID 106154)
-- Name: link; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE link (
    obj_1 integer DEFAULT 0,
    obj_2 integer DEFAULT 0,
    tbl text DEFAULT '',
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 23 (OID 106164)
-- Name: plpgsql_call_handler (); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_call_handler () RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


--
-- TOC entry 21 (OID 106165)
-- Name: plpgsql; Type: PROCEDURAL LANGUAGE; Schema: public; Owner: 
--

CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler;


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 14 (OID 120209)
-- Name: part_publication_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX part_publication_no_i ON part USING btree (publication_no);


--
-- TOC entry 15 (OID 120210)
-- Name: partdata_part_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX partdata_part_no_i ON partdata USING btree (part_no);


--
-- TOC entry 16 (OID 120211)
-- Name: publication_author_author_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX publication_author_author_no_i ON publication_author USING btree (author_no);


--
-- TOC entry 17 (OID 120212)
-- Name: publication_author_book_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX publication_author_book_no_i ON publication_author USING btree (publication_no);


--
-- TOC entry 18 (OID 120213)
-- Name: part_author_author_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX part_author_author_no_i ON part_author USING btree (author_no);


--
-- TOC entry 19 (OID 120214)
-- Name: part_author_part_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX part_author_part_no_i ON part_author USING btree (part_no);


--
-- TOC entry 20 (OID 120215)
-- Name: note_publication_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX note_publication_no_i ON note USING btree (publication_no);


