--
-- PostgreSQL database dump
--

SET SESSION AUTHORIZATION 'postgres';

SET search_path = public, pg_catalog;

--
-- TOC entry 46 (OID 17287)
-- Name: plpgsql_call_handler(); Type: FUNC PROCEDURAL LANGUAGE; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


SET SESSION AUTHORIZATION DEFAULT;

--
-- TOC entry 45 (OID 17288)
-- Name: plpgsql; Type: PROCEDURAL LANGUAGE; Schema: public; Owner: 
--

CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler;


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 3 (OID 2200)
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- TOC entry 4 (OID 300523)
-- Name: kaspa_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE kaspa_seq
    INCREMENT BY 1
    MAXVALUE 2147483647
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 5 (OID 300525)
-- Name: author; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE author (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    lastname text DEFAULT ''::text,
    firstname text DEFAULT ''::text,
    pseudonym text DEFAULT ''::text,
    born text DEFAULT ''::text,
    died text DEFAULT ''::text,
    country text DEFAULT ''::text,
    memo text DEFAULT ''::text,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 6 (OID 300542)
-- Name: part; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE part (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    publication_no integer DEFAULT 0,
    partno smallint DEFAULT 1,
    title text DEFAULT ''::text,
    pages text DEFAULT ''::text,
    memo text DEFAULT ''::text,
    "key" text DEFAULT ''::text,
    "language" text DEFAULT ''::text,
    bibtex text DEFAULT ''::text,
    intro_no integer DEFAULT 0,
    createbibtex boolean DEFAULT false,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 7 (OID 300562)
-- Name: partdata; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE partdata (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    part_no integer DEFAULT 0,
    astext text DEFAULT ''::text,
    filename text DEFAULT ''::text,
    "type" text DEFAULT ''::text,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 8 (OID 300577)
-- Name: publisher; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE publisher (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    name text DEFAULT ''::text,
    city text DEFAULT ''::text,
    serie text DEFAULT ''::text,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 9 (OID 300590)
-- Name: journal; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE journal (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    name text DEFAULT ''::text,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 10 (OID 300601)
-- Name: publication; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE publication (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    title text DEFAULT ''::text,
    subtitle text DEFAULT ''::text,
    edition smallint DEFAULT 0,
    editor text DEFAULT ''::text,
    howpublished text DEFAULT ''::text,
    organization text DEFAULT ''::text,
    publisher_no integer DEFAULT 0,
    journal_no integer DEFAULT 0,
    "year" text DEFAULT ''::text,
    pages text DEFAULT ''::text,
    translator text DEFAULT ''::text,
    volume text DEFAULT ''::text,
    number text DEFAULT ''::text,
    "month" text DEFAULT ''::text,
    series text DEFAULT ''::text,
    "type" text DEFAULT ''::text,
    "key" text DEFAULT ''::text,
    orgtitle text DEFAULT ''::text,
    orgpublisher_no integer DEFAULT 0,
    orgyear text DEFAULT ''::text,
    isbn_issn text DEFAULT ''::text,
    hidingplace text DEFAULT ''::text,
    bibtex text DEFAULT ''::text,
    entrytype smallint DEFAULT 0,
    memo text DEFAULT 0,
    createbibtex boolean DEFAULT false,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 11 (OID 300637)
-- Name: publication_author; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE publication_author (
    publication_no integer DEFAULT 0,
    author_no integer DEFAULT 0,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    author_order integer DEFAULT 0
);


--
-- TOC entry 12 (OID 300644)
-- Name: part_author; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE part_author (
    part_no integer DEFAULT 0,
    author_no integer DEFAULT 0,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    author_order integer DEFAULT 0
);


--
-- TOC entry 13 (OID 300651)
-- Name: note; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE note (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    title text DEFAULT ''::text,
    memo text DEFAULT 0,
    "type" smallint DEFAULT 0,
    publication_no integer DEFAULT 0,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    parent integer DEFAULT 0,
    sibling integer DEFAULT 0
);


--
-- TOC entry 14 (OID 300667)
-- Name: link; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE link (
    obj_1 integer DEFAULT 0,
    obj_2 integer DEFAULT 0,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text),
    tab_1 integer,
    tab_2 integer
);


SET SESSION AUTHORIZATION 'jan';

--
-- TOC entry 15 (OID 300684)
-- Name: search_wordlist; Type: TABLE; Schema: public; Owner: jan
--

CREATE TABLE search_wordlist (
    string text
);


--
-- TOC entry 16 (OID 300692)
-- Name: search_stopword; Type: TABLE; Schema: public; Owner: jan
--

CREATE TABLE search_stopword (
    word text
);


--
-- TOC entry 17 (OID 300697)
-- Name: search_index; Type: TABLE; Schema: public; Owner: jan
--

CREATE TABLE search_index (
    string oid,
    id integer,
    tab integer
);


--
-- TOC entry 18 (OID 300699)
-- Name: search_parent; Type: TABLE; Schema: public; Owner: jan
--

CREATE TABLE search_parent (
    id oid,
    parent oid
);


SET check_function_bodies = false;

--
-- TOC entry 47 (OID 300702)
-- Name: tabid(character varying); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION tabid(character varying) RETURNS integer
    AS '
declare
	tab alias for $1;
begin
  if tab=''author'' then
	return 1;
  elsif tab=''publication'' then
	return 2;
  elsif tab=''part'' then
	return 3;
  elsif tab=''partdata'' then
	return 4;
  elsif tab=''file'' then
	return 5;
  elsif tab=''note'' then
	return 6;
  elsif tab=''link'' then
	return 7;
  elsif tab=''journal'' then
	return 8;
  elsif tab=''publisher'' then
	return 9;
  elsif tab=''project'' then
	return 10;
  elsif tab=''search_index'' then
	return 11;
  elsif tab=''search_parent'' then
	return 12;
  elsif tab=''search_stopword'' then
	return 13;
  elsif tab=''search_wordlist'' then
	return 14;
  end if;
  raise EXCEPTION ''tabid(): Unknown table "%"'', tab;
  return NULL;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 19 (OID 300703)
-- Name: file; Type: TABLE; Schema: public; Owner: jan
--

CREATE TABLE file (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    partdata_no integer DEFAULT 0,
    chunk text DEFAULT ''::text,
    chunk_no integer DEFAULT 0
);


--
-- TOC entry 20 (OID 300714)
-- Name: project; Type: TABLE; Schema: public; Owner: jan
--

CREATE TABLE project (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    title text DEFAULT ''::text,
    memo text DEFAULT ''::text,
    created timestamp with time zone DEFAULT "timestamp"('now'::text),
    modified timestamp with time zone DEFAULT "timestamp"('now'::text)
);


--
-- TOC entry 21 (OID 300726)
-- Name: project_bibliography; Type: TABLE; Schema: public; Owner: jan
--

CREATE TABLE project_bibliography (
    "no" integer DEFAULT nextval('kaspa_seq'::text) NOT NULL,
    project_no integer DEFAULT 0,
    tab_no integer DEFAULT 0,
    id_no integer DEFAULT 0
);


--
-- TOC entry 49 (OID 7025869)
-- Name: create_link(character varying, character varying, character varying, character varying); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_link(character varying, character varying, character varying, character varying) RETURNS boolean
    AS '
declare
	tab1 alias for $1;
	where1 alias for $2;
	tab2 alias for $3;
	where2 alias for $4;
	obj1 integer;
	obj2 integer;
begin
  select into obj1 no from tab1 where where1;
  select into obj2 no from tab2 where where2;
  insert into link values(obj1, obj2);
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 50 (OID 7025870)
-- Name: lock_tuple(oid); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION lock_tuple(oid) RETURNS boolean
    AS '
declare
	no alias for $1;
	lock integer;
begin
  select into lock user_write_lock_oid(no);
  if lock = 1 then
    return true;
  else
    return false;
  end if;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 51 (OID 7025871)
-- Name: unlock_tuple(oid); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION unlock_tuple(oid) RETURNS boolean
    AS '
declare
	no alias for $1;
	lock integer;
begin
  loop
    select into lock user_write_unlock_oid(no);
    if lock = 0 then
      return true;
    end if;
  end loop;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 52 (OID 7025872)
-- Name: del_obj(integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_obj(integer) RETURNS boolean
    AS '
declare
	n alias for $1;
	test integer;
begin
  select into test no from author where no=n;
  if test>0 then
    delete from author where no=n;
    return FOUND;
  end if;
  select into test no from part where no=n;
  if test>0 then
    delete from part where no=n;
    return FOUND;
  end if;
  select into test no from note where no=n;
  if test>0 then
    delete from note where no=n;
    return FOUND;
  end if;
  select into test no from publication where no=n;
  if test>0 then
    delete from publication where no=n;
    return FOUND;
  end if;
  select into test no from partdata where no=n;
  if test>0 then
    delete from partdata where no=n;
    return FOUND;
  end if;
  select into test no from publisher where no=n;
  if test>0 then
    delete from publisher where no=n;
    return FOUND;
  end if;
  select into test no from journal where no=n;
  if test>0 then
    delete from journal where no=n;
    return FOUND;
  end if;
  return false;  
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 53 (OID 7025873)
-- Name: del_publication_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_publication_trigger() RETURNS "trigger"
    AS '
declare
	lock bool;
	child_locked bool;
	p record;
begin
  child_locked:=false;
  raise notice ''del_publication_trigger no=%...'',OLD.no;
  select into lock lock_tuple(OLD.oid);
  if lock = true then

    raise notice ''Deleting part publication_no= %'', OLD.no;
    for p in select no from part where publication_no=OLD.no loop
	delete from part where no=p.no;
    	if FOUND=false then
		raise notice ''parts are locked!'';
		perform unlock_tuple(OLD.oid);
		child_locked:=true;
    	end if;
    end loop;

    if child_locked=false then
	raise notice ''Deleting publication_author %'', OLD.no;
	delete from publication_author where publication_no=OLD.no;
    	raise notice ''Deleting links %'', OLD.no;
    	delete from link where (obj_1=OLD.no and tab_1=tabid(''publication'')) or (obj_2=OLD.no and tab_2=tabid(''publication''));
    	return OLD;
    end if;
  end if;
  return NULL;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 54 (OID 7025875)
-- Name: del_part_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_part_trigger() RETURNS "trigger"
    AS '
declare
	lock bool;
        p record;
        child_locked bool;
begin
  child_locked:=false;
  select into lock lock_tuple(OLD.oid);
  if lock = true then

    raise notice ''Deleting partdata part_no= %'', OLD.no;
    for p in select no from partdata where part_no=OLD.no loop
	delete from partdata where no=p.no;
    	if FOUND=false then
		raise notice ''partdatas are locked!'';
		perform unlock_tuple(OLD.oid);
		child_locked:=true;
    	end if;
    end loop;

    if child_locked=false then
	delete from link where (obj_1=OLD.no and tab_1=tabid(''part'')) or (obj_2=OLD.no and tab_2=tabid(''part''));
   	delete from part_author where part_no=OLD.no;
   	return OLD;
    end if;
  end if;
  return NULL;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 55 (OID 7025877)
-- Name: del_partdata_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_partdata_trigger() RETURNS "trigger"
    AS '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  raise notice ''del_partdata_trigger: no=%, lock=%'', OLD.no,lock;
  if lock = true then
    delete from file where partdata_no=OLD.no;
    delete from link where (obj_1=OLD.no and tab_1=tabid(''partdata'')) or (obj_2=OLD.no and tab_2=tabid(''partdata''));
    return OLD;
  else
    return NULL;
  end if;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 81 (OID 7025879)
-- Name: del_author_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_author_trigger() RETURNS "trigger"
    AS '
declare
lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    delete from publication_author where author_no=OLD.no;
    delete from part_author where author_no=OLD.no;
    delete from link where (obj_1=OLD.no and tab_1=tabid(''author'')) or (obj_2=OLD.no and tab_2=tabid(''author''));
    return OLD;
  else
    return NULL;
  end if;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 56 (OID 7025881)
-- Name: del_project_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_project_trigger() RETURNS "trigger"
    AS '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    delete from project_bibliography where project_no=OLD.no;
    return OLD;
  else
    return NULL;
  end if;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 57 (OID 7025883)
-- Name: del_publisher_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_publisher_trigger() RETURNS "trigger"
    AS '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    return OLD;
  else
    return NULL;
  end if;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 58 (OID 7025885)
-- Name: del_journal_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_journal_trigger() RETURNS "trigger"
    AS '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    return OLD;
  else
    return NULL;
  end if;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 59 (OID 7025887)
-- Name: del_note_trigger(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION del_note_trigger() RETURNS "trigger"
    AS '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    delete from link where (obj_1=OLD.no and tab_1=tabid(''note'')) or (obj_2=OLD.no and tab_2=tabid(''note''));
    return OLD;
  else
    return NULL;
  end if;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 60 (OID 7025889)
-- Name: create_author(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_author() RETURNS integer
    AS '
declare
	auno integer;
	id integer;
begin
	insert into author (lastname) values ('''');
	get diagnostics id=result_oid;
	select into auno no from author where oid=id;
	return auno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 61 (OID 7025890)
-- Name: create_publication(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_publication() RETURNS integer
    AS '
declare
	pno integer;
	id integer;
begin
	insert into publication (title) values ('''');
	get diagnostics id=result_oid;
	select into pno no from publication where oid=id;
	return pno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 62 (OID 7025891)
-- Name: create_publisher(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_publisher() RETURNS integer
    AS '
declare
	pno integer;
	id integer;
begin
	insert into publisher (name) values ('''');
	get diagnostics id=result_oid;
	select into pno no from publisher where oid=id;
	return pno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 63 (OID 7025892)
-- Name: create_project(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_project() RETURNS integer
    AS '
declare
	pno integer;
	id integer;
begin
	insert into project (title) values ('''');
	get diagnostics id=result_oid;
	select into pno no from project where oid=id;
	return pno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 64 (OID 7025893)
-- Name: create_journal(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_journal() RETURNS integer
    AS '
declare
	jno integer;
	id integer;
begin
	insert into journal (name) values ('''');
	get diagnostics id=result_oid;
	select into jno no from journal where oid=id;
	return jno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 65 (OID 7025894)
-- Name: create_part(integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_part(integer) RETURNS integer
    AS '
declare
	publ alias for $1;
	pno integer;
	id integer;
begin
	insert into part (publication_no) values (publ);
	get diagnostics id=result_oid;
	select into pno no from part where oid=id;
	return pno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 66 (OID 7025895)
-- Name: create_partdata(integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_partdata(integer) RETURNS integer
    AS '
declare
	part alias for $1;
	pdno integer;
	id integer;
begin
	insert into partdata (part_no) values (part);
	get diagnostics id=result_oid;
	select into pdno no from partdata where oid=id;
	return pdno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 67 (OID 7025896)
-- Name: create_publ_note(integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_publ_note(integer) RETURNS integer
    AS '
declare
	publ alias for $1;
	pno integer;
	id integer;
begin
	insert into note (publication_no, type) values (publ,1);
	get diagnostics id=result_oid;
	select into pno no from note where oid=id;
	return pno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 48 (OID 7025897)
-- Name: create_toplevel_note(integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION create_toplevel_note(integer) RETURNS integer
    AS '
declare
	par alias for $1;
	pno integer;
	id integer;
begin
	insert into note (parent, type) values (par,0);
	get diagnostics id=result_oid;
	select into pno no from note where oid=id;
	raise notice ''create_toplevel_note(%1) returns %2'', par, pno;
	return pno;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 69 (OID 7025906)
-- Name: search_term(character varying); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION search_term(character varying) RETURNS SETOF record
    AS '
declare
	term alias for $1;
	rec1 record;
	rec2 record;
	rec3 record;
	s varchar;
begin
	select into s lower(term)||''%'';
	raise notice ''s=%'',s;
	for rec1 in select w.oid from search_wordlist w where w.string like s loop
		RAISE NOTICE ''rec1=%'',rec1.oid;
		for rec2 in select * from search_parents(rec1.oid) union select rec1.oid as search_parents loop
			raise notice ''rec2=%'', rec2.search_parents;
			for rec3 in select text(i.id) as a, text(i.tab) as b from search_index i where i.string=rec2.search_parents loop
				raise notice ''rec3=%'', rec3.a;
				return next rec3;
			end loop;
		end loop;
	end loop;
	return;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 70 (OID 7025907)
-- Name: search_parents(oid); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION search_parents(oid) RETURNS SETOF integer
    AS '
declare
child alias for $1;
rec1 record;
rec2 record;
s text;
begin
for rec1 in select parent from search_parent where id=child loop
select into s string from search_wordlist where oid=rec1.parent;
raise NOTICE ''parent=% id=%'', s, child;
return next rec1.parent;
for rec2 in select * from search_parents(rec1.parent) loop
	return next rec2.search_parents;
end loop;
end loop;
return;
end;
'
    LANGUAGE plpgsql;


--
-- TOC entry 68 (OID 7025920)
-- Name: user_lock(integer, integer, integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_lock(integer, integer, integer) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_lock'
    LANGUAGE c;


--
-- TOC entry 71 (OID 7025921)
-- Name: user_unlock(integer, integer, integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_unlock(integer, integer, integer) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_unlock'
    LANGUAGE c;


--
-- TOC entry 72 (OID 7025922)
-- Name: user_write_lock(integer, integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_lock(integer, integer) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_lock'
    LANGUAGE c;


--
-- TOC entry 73 (OID 7025923)
-- Name: user_write_unlock(integer, integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_unlock(integer, integer) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_unlock'
    LANGUAGE c;


--
-- TOC entry 74 (OID 7025924)
-- Name: user_write_lock(integer, oid); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_lock(integer, oid) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_lock'
    LANGUAGE c;


--
-- TOC entry 75 (OID 7025925)
-- Name: user_write_unlock(integer, oid); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_unlock(integer, oid) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_unlock'
    LANGUAGE c;


--
-- TOC entry 76 (OID 7025926)
-- Name: user_write_lock_oid(oid); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_lock_oid(oid) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_lock_oid'
    LANGUAGE c;


--
-- TOC entry 77 (OID 7025927)
-- Name: user_write_unlock_oid(oid); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_unlock_oid(oid) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_unlock_oid'
    LANGUAGE c;


--
-- TOC entry 78 (OID 7025928)
-- Name: user_write_lock_oid(integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_lock_oid(integer) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_lock_oid'
    LANGUAGE c;


--
-- TOC entry 79 (OID 7025929)
-- Name: user_write_unlock_oid(integer); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_write_unlock_oid(integer) RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_write_unlock_oid'
    LANGUAGE c;


--
-- TOC entry 80 (OID 7025930)
-- Name: user_unlock_all(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION user_unlock_all() RETURNS integer
    AS '/home/jan/literature/userlocks/user_locks.so', 'user_unlock_all'
    LANGUAGE c;


--
-- TOC entry 82 (OID 7187955)
-- Name: fti(); Type: FUNCTION; Schema: public; Owner: jan
--

CREATE FUNCTION fti() RETURNS "trigger"
    AS '/home/jan/literature/fulltextindex/fti.so', 'fti'
    LANGUAGE c;


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 24 (OID 300677)
-- Name: part_publication_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX part_publication_no_i ON part USING btree (publication_no);


--
-- TOC entry 25 (OID 300678)
-- Name: partdata_part_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX partdata_part_no_i ON partdata USING btree (part_no);


--
-- TOC entry 30 (OID 300679)
-- Name: publication_author_author_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX publication_author_author_no_i ON publication_author USING btree (author_no);


--
-- TOC entry 31 (OID 300680)
-- Name: publication_author_book_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX publication_author_book_no_i ON publication_author USING btree (publication_no);


--
-- TOC entry 32 (OID 300681)
-- Name: part_author_author_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX part_author_author_no_i ON part_author USING btree (author_no);


--
-- TOC entry 33 (OID 300682)
-- Name: part_author_part_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX part_author_part_no_i ON part_author USING btree (part_no);


--
-- TOC entry 35 (OID 300683)
-- Name: note_publication_no_i; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX note_publication_no_i ON note USING btree (publication_no);


SET SESSION AUTHORIZATION 'jan';

--
-- TOC entry 36 (OID 300691)
-- Name: search_wordlist_index; Type: INDEX; Schema: public; Owner: jan
--

CREATE INDEX search_wordlist_index ON search_wordlist USING btree (string);

ALTER TABLE search_wordlist CLUSTER ON search_wordlist_index;


--
-- TOC entry 39 (OID 300701)
-- Name: search_index_str; Type: INDEX; Schema: public; Owner: jan
--

CREATE INDEX search_index_str ON search_index USING btree (string);


--
-- TOC entry 41 (OID 7025898)
-- Name: file_partdatano_index; Type: INDEX; Schema: public; Owner: jan
--

CREATE INDEX file_partdatano_index ON file USING btree (partdata_no);


--
-- TOC entry 38 (OID 7025899)
-- Name: search_index_id; Type: INDEX; Schema: public; Owner: jan
--

CREATE INDEX search_index_id ON search_index USING btree (id, tab);


--
-- TOC entry 40 (OID 7025908)
-- Name: search_parent_index_id; Type: INDEX; Schema: public; Owner: jan
--

CREATE INDEX search_parent_index_id ON search_parent USING btree (id);


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 22 (OID 300540)
-- Name: author_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY author
    ADD CONSTRAINT author_pkey PRIMARY KEY ("no");


--
-- TOC entry 23 (OID 300560)
-- Name: part_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY part
    ADD CONSTRAINT part_pkey PRIMARY KEY ("no");


--
-- TOC entry 26 (OID 300575)
-- Name: partdata_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY partdata
    ADD CONSTRAINT partdata_pkey PRIMARY KEY ("no");


--
-- TOC entry 27 (OID 300588)
-- Name: publisher_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY publisher
    ADD CONSTRAINT publisher_pkey PRIMARY KEY ("no");


--
-- TOC entry 28 (OID 300599)
-- Name: journal_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY journal
    ADD CONSTRAINT journal_pkey PRIMARY KEY ("no");


--
-- TOC entry 29 (OID 300635)
-- Name: publication_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY publication
    ADD CONSTRAINT publication_pkey PRIMARY KEY ("no");


--
-- TOC entry 34 (OID 300665)
-- Name: note_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY note
    ADD CONSTRAINT note_pkey PRIMARY KEY ("no");


SET SESSION AUTHORIZATION 'jan';

--
-- TOC entry 37 (OID 300689)
-- Name: search_wordlist_oid_key; Type: CONSTRAINT; Schema: public; Owner: jan
--

ALTER TABLE ONLY search_wordlist
    ADD CONSTRAINT search_wordlist_oid_key UNIQUE (oid);


--
-- TOC entry 42 (OID 300712)
-- Name: file_pkey; Type: CONSTRAINT; Schema: public; Owner: jan
--

ALTER TABLE ONLY file
    ADD CONSTRAINT file_pkey PRIMARY KEY ("no");


--
-- TOC entry 43 (OID 300724)
-- Name: project_pkey; Type: CONSTRAINT; Schema: public; Owner: jan
--

ALTER TABLE ONLY project
    ADD CONSTRAINT project_pkey PRIMARY KEY ("no");


--
-- TOC entry 44 (OID 300732)
-- Name: project_bibliography_pkey; Type: CONSTRAINT; Schema: public; Owner: jan
--

ALTER TABLE ONLY project_bibliography
    ADD CONSTRAINT project_bibliography_pkey PRIMARY KEY ("no");


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 90 (OID 7025874)
-- Name: del_publication_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER del_publication_trigger
    BEFORE DELETE ON publication
    FOR EACH ROW
    EXECUTE PROCEDURE del_publication_trigger();


--
-- TOC entry 85 (OID 7025876)
-- Name: del_part_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER del_part_trigger
    BEFORE DELETE ON part
    FOR EACH ROW
    EXECUTE PROCEDURE del_part_trigger();


--
-- TOC entry 87 (OID 7025878)
-- Name: del_partdata_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER del_partdata_trigger
    BEFORE DELETE ON partdata
    FOR EACH ROW
    EXECUTE PROCEDURE del_partdata_trigger();


--
-- TOC entry 84 (OID 7025880)
-- Name: del_author_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER del_author_trigger
    BEFORE DELETE ON author
    FOR EACH ROW
    EXECUTE PROCEDURE del_author_trigger();


SET SESSION AUTHORIZATION 'jan';

--
-- TOC entry 94 (OID 7025882)
-- Name: del_project_trigger; Type: TRIGGER; Schema: public; Owner: jan
--

CREATE TRIGGER del_project_trigger
    BEFORE DELETE ON project
    FOR EACH ROW
    EXECUTE PROCEDURE del_project_trigger();


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 88 (OID 7025884)
-- Name: del_publisher_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER del_publisher_trigger
    BEFORE DELETE ON publisher
    FOR EACH ROW
    EXECUTE PROCEDURE del_publisher_trigger();


--
-- TOC entry 89 (OID 7025886)
-- Name: del_journal_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER del_journal_trigger
    BEFORE DELETE ON journal
    FOR EACH ROW
    EXECUTE PROCEDURE del_journal_trigger();


--
-- TOC entry 92 (OID 7025888)
-- Name: del_note_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER del_note_trigger
    BEFORE DELETE ON note
    FOR EACH ROW
    EXECUTE PROCEDURE del_note_trigger();


--
-- TOC entry 86 (OID 7187956)
-- Name: part_fti_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER part_fti_trigger
    AFTER INSERT OR DELETE OR UPDATE ON part
    FOR EACH ROW
    EXECUTE PROCEDURE fti('memo', 'no');


--
-- TOC entry 91 (OID 7187957)
-- Name: publ_fti_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER publ_fti_trigger
    AFTER INSERT OR DELETE OR UPDATE ON publication
    FOR EACH ROW
    EXECUTE PROCEDURE fti('memo', 'no');


--
-- TOC entry 83 (OID 7187958)
-- Name: author_fti_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER author_fti_trigger
    AFTER INSERT OR DELETE OR UPDATE ON author
    FOR EACH ROW
    EXECUTE PROCEDURE fti('memo', 'no');


--
-- TOC entry 93 (OID 7187959)
-- Name: note_fti_trigger; Type: TRIGGER; Schema: public; Owner: postgres
--

CREATE TRIGGER note_fti_trigger
    AFTER INSERT OR DELETE OR UPDATE ON note
    FOR EACH ROW
    EXECUTE PROCEDURE fti('memo', 'no');


--
-- TOC entry 2 (OID 2200)
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--

COMMENT ON SCHEMA public IS 'Standard public namespace';


