-- execute after reading data:

-- Fehlermeldungen als Return
-- Nicht-Toplevelnotes knnen Waisen werden und als Zombies durch die DB flitzen.

create or replace function create_link(varchar, varchar, varchar, varchar) returns bool as '
declare
	tab1 alias for $1;
	where1 alias for $2;
	tab2 alias for $3;
	where2 alias for $4;
	obj1 integer;
	obj2 integer;
begin
  select into obj1 no from tab1 where where1;
  select into obj2 no from tab2 where where2;
  insert into link values(obj1, obj2);
end;
' language 'plpgsql';


create or replace function lock_tuple(oid) returns bool as '
declare
	no alias for $1;
	lock integer;
begin
  select into lock user_write_lock_oid(no);
  if lock = 1 then
    return true;
  else
    return false;
  end if;
end;
' language 'plpgsql';

create or replace function unlock_tuple(oid) returns bool as '
declare
	no alias for $1;
	lock integer;
begin
  loop
    select into lock user_write_unlock_oid(no);
    if lock = 0 then
      return true;
    end if;
  end loop;
end;
' language 'plpgsql';

create or replace function del_obj(integer) returns bool as '
declare
	n alias for $1;
	test integer;
begin
  select into test no from author where no=n;
  if test>0 then
    delete from author where no=n;
    return FOUND;
  end if;
  select into test no from part where no=n;
  if test>0 then
    delete from part where no=n;
    return FOUND;
  end if;
  select into test no from note where no=n;
  if test>0 then
    delete from note where no=n;
    return FOUND;
  end if;
  select into test no from publication where no=n;
  if test>0 then
    delete from publication where no=n;
    return FOUND;
  end if;
  select into test no from partdata where no=n;
  if test>0 then
    delete from partdata where no=n;
    return FOUND;
  end if;
  select into test no from publisher where no=n;
  if test>0 then
    delete from publisher where no=n;
    return FOUND;
  end if;
  select into test no from journal where no=n;
  if test>0 then
    delete from journal where no=n;
    return FOUND;
  end if;
  return false;  
end;
' language 'plpgsql';


--
-- Deletes a publication rec and the assigned part and file recs. Checks for locking.
--
create or replace function del_publication_trigger() returns trigger as '
declare
	lock bool;
	child_locked bool;
	p record;
begin
  child_locked:=false;
  raise notice ''del_publication_trigger no=%...'',OLD.no;
  select into lock lock_tuple(OLD.oid);
  if lock = true then

    raise notice ''Deleting part publication_no= %'', OLD.no;
    for p in select no from part where publication_no=OLD.no loop
	delete from part where no=p.no;
    	if FOUND=false then
		raise notice ''parts are locked!'';
		perform unlock_tuple(OLD.oid);
		child_locked:=true;
    	end if;
    end loop;

    if child_locked=false then
	raise notice ''Deleting publication_author %'', OLD.no;
	delete from publication_author where publication_no=OLD.no;
    	raise notice ''Deleting links %'', OLD.no;
    	delete from link where (obj_1=OLD.no and tab_1=tabid(''publication'')) or (obj_2=OLD.no and tab_2=tabid(''publication''));
    	return OLD;
    end if;
  end if;
  return NULL;
end;
' language 'plpgsql';

CREATE TRIGGER del_publication_trigger BEFORE DELETE ON publication FOR EACH ROW EXECUTE PROCEDURE del_publication_trigger();



create or replace function del_part_trigger() returns trigger as'
declare
	lock bool;
        p record;
        child_locked bool;
begin
  child_locked:=false;
  select into lock lock_tuple(OLD.oid);
  if lock = true then

    raise notice ''Deleting partdata part_no= %'', OLD.no;
    for p in select no from partdata where part_no=OLD.no loop
	delete from partdata where no=p.no;
    	if FOUND=false then
		raise notice ''partdatas are locked!'';
		perform unlock_tuple(OLD.oid);
		child_locked:=true;
    	end if;
    end loop;

    if child_locked=false then
	delete from link where (obj_1=OLD.no and tab_1=tabid(''part'')) or (obj_2=OLD.no and tab_2=tabid(''part''));
   	delete from part_author where part_no=OLD.no;
   	return OLD;
    end if;
  end if;
  return NULL;
end;
' language 'plpgsql';

CREATE TRIGGER del_part_trigger BEFORE DELETE ON part FOR EACH ROW EXECUTE PROCEDURE del_part_trigger();

--
--  Deletes a single file rec and checks for locking.
--
create or replace function del_partdata_trigger() returns trigger as '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  raise notice ''del_partdata_trigger: no=%, lock=%'', OLD.no,lock;
  if lock = true then
    delete from file where partdata_no=OLD.no;
    delete from link where (obj_1=OLD.no and tab_1=tabid(''partdata'')) or (obj_2=OLD.no and tab_2=tabid(''partdata''));
    return OLD;
  else
    return NULL;
  end if;
end;
' language 'plpgsql';
CREATE TRIGGER del_partdata_trigger BEFORE DELETE ON partdata FOR EACH ROW EXECUTE PROCEDURE del_partdata_trigger();



--
--  Deletes a single author rec and checks for locking.
--
create or replace function del_author_trigger() returns trigger as '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    delete from publication_author where author_no=OLD.no;
    delete from part_author where author_no=OLD.no;
    delete from link where (obj_1=OLD.no and tab_1=tabid(''author'')) or (obj_2=OLD.no and tab_2=tabid(''author''));
    return OLD;
  else
    return NULL;
  end if;
end;
' language 'plpgsql';
CREATE TRIGGER del_author_trigger BEFORE DELETE ON author FOR EACH ROW EXECUTE PROCEDURE del_author_trigger();



create or replace function del_project_trigger() returns trigger as '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    delete from project_bibliography where project_no=OLD.no;
    return OLD;
  else
    return NULL;
  end if;
end;
' language 'plpgsql';
CREATE TRIGGER del_project_trigger BEFORE DELETE ON project FOR EACH ROW EXECUTE PROCEDURE del_project_trigger();


--
--  Deletes a single publisher rec and checks for locking.
--
create or replace function del_publisher_trigger() returns trigger as '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    return OLD;
  else
    return NULL;
  end if;
end;
' language 'plpgsql';

CREATE TRIGGER del_publisher_trigger BEFORE DELETE ON publisher FOR EACH ROW EXECUTE PROCEDURE del_publisher_trigger();


--
--  Deletes a single journal rec and checks for locking.
--
create or replace function del_journal_trigger() returns trigger as '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    return OLD;
  else
    return NULL;
  end if;
end;
' language 'plpgsql';
CREATE TRIGGER del_journal_trigger BEFORE DELETE ON journal FOR EACH ROW EXECUTE PROCEDURE del_journal_trigger();



--
--  Deletes a single note rec and checks for locking.
--
create or replace function del_note_trigger() returns trigger as '
declare
	lock bool;
begin
  select into lock lock_tuple(OLD.oid);
  if lock = true then
    delete from link where (obj_1=OLD.no and tab_1=tabid(''note'')) or (obj_2=OLD.no and tab_2=tabid(''note''));
    return OLD;
  else
    return NULL;
  end if;
end;
' language 'plpgsql';
CREATE TRIGGER del_note_trigger BEFORE DELETE ON note FOR EACH ROW EXECUTE PROCEDURE del_note_trigger();



create or replace function create_author() returns integer as '
declare
	auno integer;
	id integer;
begin
	insert into author (lastname) values ('''');
	get diagnostics id=result_oid;
	select into auno no from author where oid=id;
	return auno;
end;
'	language 'plpgsql';

create or replace function create_publication() returns integer as '
declare
	pno integer;
	id integer;
begin
	insert into publication (title) values ('''');
	get diagnostics id=result_oid;
	select into pno no from publication where oid=id;
	return pno;
end;
'	language 'plpgsql';

create or replace function create_publisher() returns integer as '
declare
	pno integer;
	id integer;
begin
	insert into publisher (name) values ('''');
	get diagnostics id=result_oid;
	select into pno no from publisher where oid=id;
	return pno;
end;
'	language 'plpgsql';

create or replace function create_project() returns integer as '
declare
	pno integer;
	id integer;
begin
	insert into project (title) values ('''');
	get diagnostics id=result_oid;
	select into pno no from project where oid=id;
	return pno;
end;
'	language 'plpgsql';


create or replace function create_journal() returns integer as '
declare
	jno integer;
	id integer;
begin
	insert into journal (name) values ('''');
	get diagnostics id=result_oid;
	select into jno no from journal where oid=id;
	return jno;
end;
'	language 'plpgsql';



create or replace function create_part(integer) returns integer as '
declare
	publ alias for $1;
	pno integer;
	id integer;
begin
	insert into part (publication_no) values (publ);
	get diagnostics id=result_oid;
	select into pno no from part where oid=id;
	return pno;
end;
'	language 'plpgsql';

create or replace function create_partdata(integer) returns integer as '
declare
	part alias for $1;
	pdno integer;
	id integer;
begin
	insert into partdata (part_no) values (part);
	get diagnostics id=result_oid;
	select into pdno no from partdata where oid=id;
	return pdno;
end;
'	language 'plpgsql';


create or replace function create_publ_note(integer) returns integer as '
declare
	publ alias for $1;
	pno integer;
	id integer;
begin
	insert into note (publication_no, type) values (publ,1);
	get diagnostics id=result_oid;
	select into pno no from note where oid=id;
	return pno;
end;
'	language 'plpgsql';

create or replace function create_toplevel_note(integer) returns integer as '
declare
	par alias for $1;
	pno integer;
	id integer;
begin
	insert into note (parent, type) values (par,0);
	get diagnostics id=result_oid;
	select into pno no from note where oid=id;
	raise notice ''create_toplevel_note(%1) returns %2'', par, pno;
	return pno;
end;
'	language 'plpgsql';


create index file_partdatano_index on file(partdata_no);
create index search_index_id on search_index(id, tab);
alter table partdata drop file;
update partdata set type=type;
vacuum full analyze verbose;

-- create trigger partdata_fti_trigger after update or insert or delete on partdata for each row execute procedure fti(astext, no);

create or replace function search_term(varchar) returns setof record as '
declare
	term alias for $1;
	rec1 record;
	rec2 record;
	rec3 record;
	s varchar;
begin
	select into s lower(term)||''%'';
	raise notice ''s=%'',s;
	for rec1 in select w.oid from search_wordlist w where w.string like s loop
		RAISE NOTICE ''rec1=%'',rec1.oid;
		for rec2 in select * from search_parents(rec1.oid) union select rec1.oid as search_parents loop
			raise notice ''rec2=%'', rec2.search_parents;
			for rec3 in select text(i.id) as a, text(i.tab) as b from search_index i where i.string=rec2.search_parents loop
				raise notice ''rec3=%'', rec3.a;
				return next rec3;
			end loop;
		end loop;
	end loop;
	return;
end;
'	language 'plpgsql';

create or replace function search_parents(oid) returns setof integer as '
declare
child alias for $1;
rec1 record;
rec2 record;
s text;
begin
for rec1 in select parent from search_parent where id=child loop
select into s string from search_wordlist where oid=rec1.parent;
raise NOTICE ''parent=% id=%'', s, child;
return next rec1.parent;
for rec2 in select * from search_parents(rec1.parent) loop
	return next rec2.search_parents;
end loop;
end loop;
return;
end;
'language 'plpgsql';


create index search_parent_index_id on search_parent(id);
vacuum analyze search_parent;
cluster search_wordlist_index on search_wordlist;
cluster search_parent_index on search_parent;
cluster search_index_str on search_index;