/***************************************************************************
 *                                                                         *
 *   author.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "author.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qlayout.h>
#include <qsettings.h>
#include "error.h"
#include "editorhelper.h"
#include "global.h"
#include "linklistbox.h"
#include "filllinklistbox.h"
#include "mainwindowimpl.h"
#include "selectitem.h"
#include "selectnote.h"
#include "selectpart.h"

Author::Author(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name), toolbar(0L) {    
    authorform=new AuthorForm(this, "authorform");
    QHBoxLayout *l=new QHBoxLayout(this);
    l->addWidget(authorform);
    initToolBar();
    new EditorHelper(this, authorform->memo);
    QSettings settings;
    QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
    if(!backgr.isEmpty()) authorform->setErasePixmap(QPixmap(backgr));

    setData();
}

void Author::initToolBar() {
    toolbar=new QToolBar(mainWindow());
    
    // **** Note ****
    insertNoteAction=new QAction("Insert Note", QIconSet(loadPixmap("insnote.png")), "Insert Note", 0, this, "insertnote");
    connect(insertNoteAction, SIGNAL(activated()), this, SLOT(insertNote()));
    createNoteAction=new QAction("Create Note", QIconSet(loadPixmap("newnote.png")), "Create Note", 0, this, "createnote");
    connect(createNoteAction, SIGNAL(activated()), this, SLOT(createNote()));
    deleteNoteAction=new QAction("Delete Note", QIconSet(loadPixmap("delnote.png")), "Delete Note", 0, this, "deletenote");
    connect(deleteNoteAction, SIGNAL(activated()), this, SLOT(deleteNote()));
    
    // **** Publication ****
    insertPublicationAction=new QAction("Insert Publication", QIconSet(loadPixmap("inspubl.png")), "Insert Publication", 0, this, "insertpublication");
    connect(insertPublicationAction, SIGNAL(activated()), this, SLOT(insertPublication()));
    createPublicationAction=new QAction("Create Publication", QIconSet(loadPixmap("newpubl.png")), "Create Publication", 0, this, "createpublication");
    connect(createPublicationAction, SIGNAL(activated()), this, SLOT(createPublication()));
    deletePublicationAction=new QAction("Delete Publication", QIconSet(loadPixmap("delpubl.png")), "Delete Publication", 0, this, "deletepublication");
    connect(deletePublicationAction, SIGNAL(activated()), this, SLOT(deletePublication()));
    

    // **** Part ****
    insertPartAction=new QAction("Insert Part", QIconSet(loadPixmap("inspart.png")), "Insert Part", 0, this, "insertpart");
    connect(insertPartAction, SIGNAL(activated()), this, SLOT(insertPart()));
    deletePartAction=new QAction("Delete Part", QIconSet(loadPixmap("delpart.png")), "Delete Part", 0, this, "deletepart");
    connect(deletePartAction, SIGNAL(activated()), this, SLOT(deletePart()));

            
    insertNoteAction->addTo(toolbar);
    createNoteAction->addTo(toolbar);
    deleteNoteAction->addTo(toolbar);

    insertPublicationAction->addTo(toolbar);
    createPublicationAction->addTo(toolbar);
    deletePublicationAction->addTo(toolbar);

    insertPartAction->addTo(toolbar);
    deletePartAction->addTo(toolbar);
   
    toolbar->show();

    insertNoteAction->addTo(roMenu());
    createNoteAction->addTo(roMenu());
    deleteNoteAction->addTo(roMenu());

    insertPublicationAction->addTo(roMenu());
    createPublicationAction->addTo(roMenu());
    deletePublicationAction->addTo(roMenu());

    insertPartAction->addTo(roMenu());
    deletePartAction->addTo(roMenu());


}

Author::~Author() {
    write();
    delete toolbar;

}

bool Author::setData()
{
    try {
	ReadWriteFormBase::setData();
	LitUrl u=url();
	setWorkingGui(true);
	form.clear();
	formcursor.clear();
	formcursor.setMode(QSqlCursor::Writable);
	formcursor.setName("author");
	if(!formcursor.select(u.query()))
	    throw(Error(formcursor.lastError()));
	qDebug("count:"+QString().setNum(formcursor.size()));
	if(formcursor.size()!=1) throw Error("Invalid Record!");
	formcursor.first();
	QString no=formcursor.value("no").toString();
	form.setRecord(formcursor.primeUpdate());
	form.insert(authorform->firstname, "firstname");
	form.insert(authorform->lastname, "lastname");
	form.insert(authorform->pseudonym, "pseudonym");
	form.insert(authorform->country, "country");
	form.insert(authorform->born, "born");
	form.insert(authorform->died, "died");
	form.insert(authorform->memo, "memo");
	form.insert(authorform->modified, "modified");
	form.insert(authorform->created, "created");
	form.readFields();

	authorform->links->setEnabled(false);
	authorform->publications->setEnabled(false);
	authorform->parts->setEnabled(false);
	
	authorform->publications->clear();
	fillpubl=new FillLinkListBox(this, "publauthorfllb", authorform->publications, "select publication.no, publication.title from publication, publication_author where publication.no=publication_author.publication_no and publication_author.author_no="+no+" order by title","publication/publication", loadPixmap("showpubl.png"));
	fillpubl->start();
	threads.append(fillpubl);

	authorform->parts->clear();
	fillpart=new FillLinkListBox(this, "partauthorfllb", authorform->parts, "select part.no, part.title from part, part_author where part.no=part_author.part_no and part_author.author_no="+no+" order by part.title", "part/part", loadPixmap("showpart.png"));
	fillpart->start();
	threads.append(fillpart);

	authorform->links->clear();
	filllink=new FillLinkListBox(this, "linkauthorfllb", authorform->links, "select note.no, note.title from note, link where "+buildLinkWhereClause("note", "author", no)+" order by note.title","note/note", loadPixmap("shownote.png"));
	filllink->start();
	threads.append(filllink);

	setValid(true);
    } catch(Error &e) {
	setValid(false);
	setReadonlyGui(true);
	QMessageBox::warning(this, "Error", e.message());
	return false;
    }
    setValid(true);
    return true;
}

void Author::setWorkingGui(bool work) {
    ReadWriteFormBase::setWorkingGui(work);
    authorform->links->setEnabled(!work);
    authorform->publications->setEnabled(!work);
    authorform->parts->setEnabled(!work);
}

void Author::setReadonlyGui(bool ro) {
    debug("Author::setReadonlyGui");
    ReadWriteFormBase::setReadonlyGui(ro);    
    authorform->firstname->setEnabled(!ro);
    authorform->lastname->setEnabled(!ro);
    authorform->pseudonym->setEnabled(!ro);
    authorform->country->setEnabled(!ro);
    authorform->born->setEnabled(!ro);
    authorform->died->setEnabled(!ro);
    authorform->memo->setEnabled(!ro);
 }

void Author::setInvalidGui(bool inv) {
    ReadWriteFormBase::setInvalidGui(inv);
    setWorkingGui(false);
    setReadonlyGui(inv);
}

void Author::insertNote() {
    SelectNote *s=new SelectNote(this, "selnote");
    Q_ASSERT(s);
    if(s->exec()==QDialog::Accepted)
      createLink(s->url(), url());
}

void Author::deleteNote() {
  deleteLink(authorform->links->url(),url());    
}

void Author::createNote() {
    newNote(url());
}

void Author::insertPublication() {
    SelectItem *s=new SelectItem(this, "selpubls");
    Q_ASSERT(s);
    s->setData("publication/publication", "select no, title from publication order by title");
    if(s->exec()==QDialog::Accepted)
      createPublicationAuthor(s->url(), url());
}

void Author::deletePublication() {
    deletePublicationAuthor(authorform->publications->url(),url());
}

void Author::createPublication() {
}

void Author::insertPart() {
    SelectPart *s=new SelectPart(this, "selpart");
    Q_ASSERT(s);
    if(s->exec()==QDialog::Accepted)
      createPartAuthor(s->url(), url());
}

void Author::deletePart() {
  deletePartAuthor(authorform->parts->url(),url());
}
