/***************************************************************************
 *                                                                         *
 *   bibtexfactory.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "bibtexfactory.h"
#include <qstring.h>
#include <qsqlquery.h>
#include <qsqlcursor.h>
#include <qregexp.h>
#include "bibtextypes.h"


BibTexFactory::BibTexFactory(QSqlCursor &cur) {
}

PublBibTexFactory::PublBibTexFactory(QSqlCursor &cur): BibTexFactory(cur) {
  
  int bibtextype=cur.value("entrytype").toInt();
  QString publisher_no=cur.value("publisher_no").toString();
  QString publisher, address;
  QSqlQuery query("select publisher.name, publisher.city from publisher where no="+publisher_no);
  if(query.first()) {
    publisher=query.value(0).toString();
    address=query.value(1).toString();
  }

  QString journal_no=cur.value("journal_no").toString();
  QString journal;
  query.exec("select journal.name from journal where no="+journal_no);
  if(query.first()) 
    journal=query.value(0).toString();

  QString no=cur.value("no").toString();
  QString editor=cur.value("editor").toString();
  QString title=cur.value("title").toString();
  QString year=cur.value("year").toString();
  QString volume=cur.value("volume").toString();
  QString number=cur.value("number").toString();
  QString series=cur.value("series").toString();
  QString edition=cur.value("edition").toString();
  QString month=cur.value("month").toString();
  QString pages=cur.value("pages").toString();
  QString ptype=cur.value("type").toString();
  QString key=cur.value("key").toString();
  QString organization=cur.value("organization").toString();
  QString howpublished=cur.value("howpublished").toString();  


  query.exec("select author.firstname, author.lastname from author, publication_author where author.no=publication_author.author_no and publication_author.publication_no="+no+" order by publication_author.author_order");

  QString idname, firstname, lastname, author;
  if(query.first()) {
    idname=query.value(1).toString();
    do {
      firstname=query.value(0).toString();
      lastname=query.value(1).toString();
      author+=lastname+", "+firstname+" and ";
    } while(query.next());
    author=author.left(author.length()-5);
  } 
  else
    author="NoName"+no;


  QString label=idname+year;
  query.exec("select publication.no as no, publication.created as created from publication, publication_author, author where publication.year='"+year+"' and publication.no = publication_author.publication_no and publication_author.author_no=author.no and author.lastname='"+idname+"' union select part.no as no, part.created as created from part, part_author, author, publication where part.no = part_author.part_no and part_author.author_no=author.no and author.lastname='"+idname+"' and part.publication_no=publication.no and publication.year='"+year+"' order by created");
  int bibindex=-1;
  while(query.next() && query.value(0).toInt()!=cur.value("no").toInt())
    bibindex++;

  if(bibindex>0)
    label+=char('a'+bibindex);
    
  switch(bibtextype) {
  case BT_BOOK:
    start(label, "@BOOK");
    if(!author.isEmpty()) addAuthor(author);
    if(!editor.isEmpty()) addEditor(editor);
    if(!title.isEmpty()) addTitle(title);
    if(!publisher.isEmpty()) addPublisher(publisher);
    if(!year.isEmpty()) addYear(year);
    if(!volume.isEmpty()) addVolume(volume);
    if(!number.isEmpty()) addNumber(number);
    if(!series.isEmpty()) addSeries(series);
    if(!address.isEmpty()) addAddress(address);
    if(!edition.isEmpty()) addEdition(edition);
    if(!month.isEmpty()) addMonth(month);
    end();
    break;
  case BT_INBOOK:     // crossref-type
    start("Ref"+no, "@INBOOK");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!publisher.isEmpty()) addPublisher(publisher);
    if(!year.isEmpty()) addYear(year);
    if(!volume.isEmpty()) addVolume(volume);
    if(!number.isEmpty()) addNumber(number);
    if(!series.isEmpty()) addSeries(series);
    if(!ptype.isEmpty()) addType(ptype);
    if(!address.isEmpty()) addAddress(address);
    if(!edition.isEmpty()) addEdition(edition);
    if(!month.isEmpty()) addMonth(month);
    if(!key.isEmpty()) addKey(key);
    end();
    break;
  case BT_INCOLLECTION:     // crossref-type
    start("Ref"+no, "@INCOLLECTION");
    if(!title.isEmpty()) addBooktitle(title);
    if(!publisher.isEmpty()) addPublisher(publisher);
    if(!year.isEmpty()) addYear(year);
    if(!editor.isEmpty()) addEditor(author);
    if(!volume.isEmpty()) addVolume(volume);
    if(!number.isEmpty()) addNumber(number);
    if(!series.isEmpty()) addSeries(series);
    if(!ptype.isEmpty()) addType(ptype);
    if(!address.isEmpty()) addAddress(address);
    if(!edition.isEmpty()) addEdition(edition);
    if(!month.isEmpty()) addMonth(month);
    if(!key.isEmpty()) addKey(key);
    end();
    break;
  case BT_BOOKLET:
    start(label, "@BOOKLET");
    if(!title.isEmpty()) addTitle(title);
    if(!author.isEmpty()) addAuthor(author);
    if(!howpublished.isEmpty()) addHowpublished(howpublished);
    if(!address.isEmpty()) addAddress(address);
    if(!month.isEmpty()) addMonth(month);
    if(!year.isEmpty()) addYear(year);
    end();
    break;
  case BT_MISC:
    start(label, "@MISC");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!howpublished.isEmpty()) addHowpublished(howpublished);
    if(!month.isEmpty()) addMonth(month);
    if(!year.isEmpty()) addYear(year);
    end();
    break;
  case BT_ARTICLE:
    start(label, "@ARTICLE");
    if(!title.isEmpty()) addTitle(title);
    if(!author.isEmpty()) addAuthor(author);
    if(!journal.isEmpty()) addJournal(journal);
    if(!year.isEmpty()) addYear(year);
    if(!volume.isEmpty()) addVolume(volume);
    if(!number.isEmpty()) addNumber(number);
    if(!pages.isEmpty()) addPages(pages);
    if(!month.isEmpty()) addMonth(month);
    if(!key.isEmpty()) addKey(key);
    end();
    break;
  case BT_MANUAL:
    start(label, "@MANUAL");
    if(!title.isEmpty()) addTitle(title);
    if(!author.isEmpty()) addAuthor(author);
    if(!organization.isEmpty()) addOrganization(organization);
    if(!address.isEmpty()) addAddress(address);
    if(!edition.isEmpty()) addEdition(edition);
    if(!month.isEmpty()) addMonth(month);
    if(!year.isEmpty()) addYear(year);
    end();
    break;
  case BT_PROCEEDING:
    start(label, "@PROCEEDINGS");
    if(!title.isEmpty()) addTitle(title);
    if(!year.isEmpty()) addYear(year);
    if(!editor.isEmpty()) addEditor(editor);
    if(!volume.isEmpty()) addVolume(volume);
    if(!number.isEmpty()) addNumber(number);
    if(!series.isEmpty()) addSeries(series);
    if(!address.isEmpty()) addAddress(address);
    if(!month.isEmpty()) addMonth(month);
    if(!organization.isEmpty()) addOrganization(organization);
    if(!publisher.isEmpty()) addPublisher(publisher);
    end();
    break;
  case BT_INPROCEEDING:     // crossref-type
    start("Ref"+no, "@INPROCEEDINGS");
    if(!title.isEmpty()) addBooktitle(title);
    if(!year.isEmpty()) addYear(year);
    if(!editor.isEmpty()) addEditor(author);
    if(!volume.isEmpty()) addVolume(volume);
    if(!number.isEmpty()) addNumber(number);
    if(!series.isEmpty()) addSeries(series);
    if(!address.isEmpty()) addAddress(address);
    if(!month.isEmpty()) addMonth(month);
    if(!organization.isEmpty()) addOrganization(organization);
    if(!publisher.isEmpty()) addPublisher(publisher);
    if(!key.isEmpty()) addKey(key);
    end();
    break;
  case BT_TECHREPORT:
    start(label, "@TECHREPORT");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!organization.isEmpty()) addInstitution(organization);
    if(!year.isEmpty()) addYear(year);
    if(!address.isEmpty()) addAddress(address);
    if(!ptype.isEmpty()) addType(ptype);
    if(!number.isEmpty()) addNumber(number);
    if(!month.isEmpty()) addMonth(month);
    end();
    break;
  case BT_UNPUBLISHED:
    start(label, "@UNPUBLISHED");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!month.isEmpty()) addMonth(month);
    if(!year.isEmpty()) addYear(year);
    end();
    break;
  case BT_PHDTHESIS:
    start(label, "@PHDTHESIS");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!organization.isEmpty()) addSchool(organization);
    if(!year.isEmpty()) addYear(year);
    if(!ptype.isEmpty()) addType(ptype);
    if(!address.isEmpty()) addAddress(address);
    if(!month.isEmpty()) addMonth(month);
    end();
    break;
  case BT_MASTERTHESIS:
    start(label, "@MASTERSTHESIS");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!organization.isEmpty()) addSchool(organization);
    if(!year.isEmpty()) addYear(year);
    if(!ptype.isEmpty()) addType(ptype);
    if(!address.isEmpty()) addAddress(address);
    if(!month.isEmpty()) addMonth(month);
    end();
    break;
    
  }
}


PartBibTexFactory::PartBibTexFactory(QSqlCursor &cur): BibTexFactory(cur) {

  QString no=cur.value("no").toString();
  int entrytype;
  QString year;
  QString publno;
  QSqlQuery query("select publication.entrytype, publication.year, publication.no from publication, part where publication.no=part.publication_no and part.no="+no);
  if(query.first()) {
    entrytype=query.value(0).toInt();
    year=query.value(1).toString();
    publno=query.value(2).toString();
  }


  QString title=cur.value("title").toString();
  QString key=cur.value("key").toString();
  QString pages=cur.value("pages").toString();
  QString chapter=cur.value("partno").toString();  
  QString crossref="Ref"+publno;

  
  query.exec("select author.firstname, author.lastname from author, part_author where author.no=part_author.author_no and part_author.part_no="+no+" order by part_author.author_order");

  QString idname, firstname, lastname, author;
  if(query.first()) {
    idname=query.value(1).toString();
    do {
      firstname=query.value(0).toString();
      lastname=query.value(1).toString();
      author+=lastname+", "+firstname+" and ";
    } while(query.next());
    author=author.left(author.length()-5);
  } 
  else
    author="NoName"+cur.value("no").toString();


  QString label=idname+year;
  query.exec("select publication.no as no, publication.created as created from publication, publication_author, author where publication.year='"+year+"' and publication.no = publication_author.publication_no and publication_author.author_no=author.no and author.lastname='"+idname+"' union select part.no as no, part.created as created from part, part_author, author, publication where part.no = part_author.part_no and part_author.author_no=author.no and author.lastname='"+idname+"' and part.publication_no=publication.no and publication.year='"+year+"' order by created");
  int bibindex=-1;
  while(query.next() && query.value(0).toInt()!=cur.value("no").toInt())
    bibindex++;

  if(bibindex>0)
    label+=char('a'+bibindex);
    
  switch(entrytype) {
  case BT_INBOOK:
    start(label, "@INBOOK");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!chapter.isEmpty()) addChapter(chapter);
    if(!pages.isEmpty()) addPages(pages);
    if(!key.isEmpty()) addKey(key);
    if(!crossref.isEmpty()) addCrossRef(crossref);
    end();
    break;
  case BT_INCOLLECTION:
    start(label, "@INCOLLECTION");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!chapter.isEmpty()) addChapter(chapter);
    if(!pages.isEmpty()) addPages(pages);
    if(!key.isEmpty()) addKey(key);
    if(!crossref.isEmpty()) addCrossRef(crossref);
    end();
    break;
  case BT_INPROCEEDING:
    start(label, "@INPROCEEDINGS");
    if(!author.isEmpty()) addAuthor(author);
    if(!title.isEmpty()) addTitle(title);
    if(!pages.isEmpty()) addPages(pages);
    if(!key.isEmpty()) addKey(key);
    if(!crossref.isEmpty()) addCrossRef(crossref);
    end();
    break;
  }
}

QString BibTexFactory::getEntry() {
  return entry;
}

QString BibTexFactory::ascii2latex(QString &t) {
  QString s(t);
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"A}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"E}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"I}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"O}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"U}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"a}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"e}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"i}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"o}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"u}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"y}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\AE}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\ae}");
  s.replace(QRegExp(QString().fromLatin1("")),"{\\\"s}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{A}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{E}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{I}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{O}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{U}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{Y}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{a}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{e}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{i}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{o}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{u}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\'{y}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{A}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{E}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{I}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{O}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{U}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{a}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{e}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{i}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{o}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\`{u}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{A}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{E}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{I}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{O}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{U}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{a}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{e}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{i}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{o}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\^{u}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\~{A}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\~{N}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\~{O}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\~{a}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\~{n}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\~{o}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\r{A}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\r{a}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\c{C}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\c{c}");
  s.replace(QRegExp(QString().fromLatin1("")),"\\O");
  s.replace(QRegExp(QString().fromLatin1("")),"\\o");
  return s;
}

void BibTexFactory::start(QString label, QString ptype) {  entry=ptype+"{"+label;  }
 void BibTexFactory::end() { entry +="\n}"; }
 void BibTexFactory::addAddress(QString &s) { entry+=",\naddress=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addAuthor(QString &s) { entry+=",\nauthor=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addBooktitle(QString &s) { entry+=",\nbooktitle=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addChapter(QString &s) { entry+=",\nchapter=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addEdition(QString &s) { entry+=",\nedition=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addEditor(QString &s) { entry+=",\neditor=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addHowpublished(QString &s) { entry+=",\nhowpublished=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addInstitution(QString &s) { entry+=",\ninstitution=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addJournal(QString &s) { entry+=",\njournal=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addMonth(QString &s) { entry+=",\nmonth=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addNote(QString &s) { entry+=",\nnote=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addNumber(QString &s) { entry+=",\nnumber="+s; }
 void BibTexFactory::addOrganization(QString &s){ entry+=",\norganization=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addPages(QString &s) { entry+=",\npages=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addPublisher(QString &s) { entry+=",\npublisher=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addSchool(QString &s) { entry+=",\nschool=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addSeries(QString &s) { entry+=",\nseries=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addTitle(QString &s) { entry+=",\ntitle=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addType(QString &s) { entry+=",\ntype=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addVolume(QString &s) { entry+=",\nvolume="+s; }
 void BibTexFactory::addYear(QString &s) { entry+=",\nyear="+s; }
 void BibTexFactory::addKey(QString &s) { entry+=",\nkey=\""+ascii2latex(s)+"\""; }
 void BibTexFactory::addCrossRef(QString &s) { entry+=",\ncrossref=\""+ascii2latex(s)+"\""; }
 
