/***************************************************************************
 *                                                                         *
 *   convert2text.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "convert2text.h"
#include <stdio.h>
#include <kprocess.h>
#include <kglobal.h>
#include <kconfig.h>
#include <qregexp.h>
#include "error.h"
#include <qsettings.h>
#include <qapplication.h>
#include <kmimemagic.h>


Convert2Text::Convert2Text(){
  //  tmpfiles.setAutoDelete(true);
}

Convert2Text::~Convert2Text(){
  /*
    for(unsigned int i=0; i<tmpfiles.count(); i++) {
    tmpfiles.at(i)->remove();
    }
  */
}


QString Convert2Text::buf2file(QByteArray b) {
  QString fname=tmpnam(0L);
  
  QFile *f=new QFile(fname);
  if (!f->open(IO_WriteOnly) )
    throw Error("Kann kein temporres File ffnen!");
  f->writeBlock(b);
  f->close();
  tmpfiles.append(f);
  return fname;
}

void Convert2Text::slotReceivedStderr(KProcess* proc,char* buf,int len) {
  buf[len-1]=0;
  debug("Stderr:"+QString(buf)+"\n");
  emit output(QString(QCString(buf, len)));
}

void Convert2Text::slotReceivedStdout(KProcess* proc,char* buf,int len) {
  buf[len-1]=0;
  debug("Stdout:"+QString(buf)+"\n");
  emit output(QString(QCString(buf, len)));
}

QString Convert2Text::convert(QString cmd, QString infilename) {
  //  cout <<"Convert2Text()\n";

  QString outfilename=tmpnam(0L);
  KShellProcess proc;
  char *c=new char[200];
  cmd.replace(QRegExp("#in#"), infilename);
  cmd.replace(QRegExp("#out#"), outfilename);

  connect(&proc, SIGNAL(receivedStderr (KProcess*,char*,int)),
          this, SLOT(slotReceivedStderr (KProcess*,char*,int)) );
  connect(&proc, SIGNAL(receivedStdout (KProcess*,char*,int)),
          this, SLOT(slotReceivedStdout (KProcess*,char*,int)) );

  debug("Executing \""+cmd+"\"\n");
  proc << cmd;
  //  proc<<"html2text /tmp/test.html > /tmp/test.txt";
  if(!proc.start(KProcess::DontCare, KProcess::All))
    throw Error("Could not start process!");
  
  while(proc.isRunning()) {
    qApp->processEvents();
  }
  
  debug("Convert2Text, exitstatus:"+QString().setNum(proc.exitStatus())+"\n");
  
  if(!proc.normalExit())
    throw Error("Conversion process: Abnormal termination!");

  QFile f(outfilename);
  //  QFile f("/tmp/test.txt");
  if(!f.open(IO_ReadOnly))
    throw Error("Can't open temporary file!");

  delete c;
  QByteArray b=f.readAll();
  f.close();
  unlink(outfilename);
  unlink(infilename);

  return QString(b);
}

bool Convert2Text::mime2text(QString infilename, QString &out) {
  QSettings settings;


#ifdef HAS_KDE
  KMimeMagicResult *mime=KMimeMagic::self()->findFileType(infilename);
  QString mimetype=mime->mimeType();
#endif

  settings.setPath("www.test.de", "Literature");
  bool ok;
  debug("Convert2Text::mime2text - "+mimetype);
  QString s=settings.readEntry("/literature/mimetotext/"+mimetype, QString::null, &ok);
  debug("Befehl ist: "+s);
  if(!ok) return false;

  if(s=="#direct#") {
    QFile f("/tmp/test_convert");
    f.open(IO_ReadOnly);
    QString b=f.readAll();
    f.close();
    return QString(b);
  }

  out=QString(convert(s, infilename));
  return true;
}
