/***************************************************************************
 *                                                                         *
 *   editorhelper.h								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#ifndef EditorHelper_included
#define EditorHelper_included

#include <qobject.h>
#include <qtextedit.h>

class QAction;
class QComboBox;
class QString;
class QFont;
class QColor;
class QToolBar;
class ReadWriteFormBase;


class EditorHelper: public QObject
{
    Q_OBJECT


private:
	EditorHelper(const EditorHelper&);
	EditorHelper& operator=(const EditorHelper&);
	
public:
  EditorHelper(ReadWriteFormBase *parent, QTextEdit *editor);
  ~EditorHelper();
  
private:
    QAction *actionTextBold,
            *actionTextUnderline,
            *actionTextItalic,
            *actionTextColor,
            *actionAlignLeft,
            *actionAlignCenter,
            *actionAlignRight,
            *actionAlignJustify;
    QComboBox *comboStyle,
              *comboFont,
              *comboSize;

   ReadWriteFormBase *parent;
   QTextEdit *e;
   QToolBar *toolbar;


public slots:
    void setupToolBar();
    void setupEditorConnections();

private slots:
    void editUndo();
    void editRedo();
    void editCut();
    void editCopy();
    void editPaste();

    void textBold();
    void textUnderline();
    void textItalic();
    void textFamily( const QString &f );
    void textSize( const QString &p );
    void textStyle( int s );
    void textColor();
    void textAlign( QAction *a );

    void fontChanged( const QFont &f );
    void colorChanged( const QColor &c );
    void alignmentChanged( int a );
};
#endif
