/***************************************************************************
 *                                                                         *
 *   file.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "file.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qlayout.h>
#include <qsettings.h>
#include <qprogressdialog.h>
#include "error.h"
#include "editorhelper.h"
#include "global.h"
#include "linklistbox.h"
#include "filllinklistbox.h"
#include "mainwindowimpl.h"
#include "selectitem.h"
#include "selectnote.h"
#include <qfile.h>
#include <qsplitter.h>

#ifdef HAS_KDE
#include <part.h>
#include <ktrader.h>
#include <kmimemagic.h>
#include <kservice.h>
#include <kmdcodec.h>
#include <kio/netaccess.h>
#include <klibloader.h>
#include <rundialog.h>
#include <kmdcodec.h>
#endif

File::File(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name), toolbar(0L), part(0L) {    
    fileform=new FileForm(this, "fileform");
    QHBoxLayout *l=new QHBoxLayout(this);
    l->addWidget(fileform);
    initToolBar();
    new EditorHelper(this, fileform->memo);
    QSettings settings;
    QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
    if(!backgr.isEmpty()) fileform->setErasePixmap(QPixmap(backgr));
    setData();
}

void File::initToolBar() {
    toolbar=new QToolBar(mainWindow());
    
    insertNoteAction=new QAction("Insert Note", QIconSet(loadPixmap("insnote.png")), "Insert Note", 0, this, "insertnote");
    connect(insertNoteAction, SIGNAL(activated()), this, SLOT(insertNote()));

    createNoteAction=new QAction("Create Note", QIconSet(loadPixmap("newnote.png")), "Create Note", 0, this, "createnote");
    connect(createNoteAction, SIGNAL(activated()), this, SLOT(createNote()));

    deleteNoteAction=new QAction("Delete Note", QIconSet(loadPixmap("delnote.png")), "Delete Note", 0, this, "deletenote");
    connect(deleteNoteAction, SIGNAL(activated()), this, SLOT(deleteNote()));

    runApplicationAction=new QAction("Run Application", QIconSet(loadPixmap("run.png")), "Run Application", 0, this, "runapplication");
    connect(runApplicationAction, SIGNAL(activated()), this, SLOT(runApp()));

    addToProjectAction=new QAction("Add to Project", QIconSet(loadPixmap("gnome-folder.png")), "Add to Project", 0, this, "addtoproject");
    connect(addToProjectAction, SIGNAL(activated()), this, SLOT(addToProject()));

    insertNoteAction->addTo(toolbar);
    createNoteAction->addTo(toolbar);
    deleteNoteAction->addTo(toolbar);
    runApplicationAction->addTo(toolbar);
    addToProjectAction->addTo(toolbar);

    toolbar->show();

    insertNoteAction->addTo(roMenu());
    createNoteAction->addTo(roMenu());
    deleteNoteAction->addTo(roMenu());
    runApplicationAction->addTo(roMenu());
    addToProjectAction->addTo(roMenu());
}

File::~File() {
    write();
    QSqlQuery query("select part_no from partdata where no="+no(url()));
    query.first();
    QString partno=query.value(0).toString();
    query.prepare("update part set memo=? where no="+partno);
    query.bindValue(0, fileform->memo->text());
    query.exec();
    delete toolbar;
}

bool File::setData()
{
  try {
    ReadWriteFormBase::setData();
    LitUrl u=url();
    setWorkingGui(true);

    QSqlQuery query("select part.memo from part, partdata where part.no=partdata.part_no and partdata.no="+no(url()));
    Q_ASSERT(query.first());
    fileform->memo->setText(query.value(0).toString());

    form.clear();
    formcursor.clear();
    formcursor.setMode(QSqlCursor::Writable);
    formcursor.setName("partdata");

    if(!formcursor.select(u.query()))
      throw(Error(formcursor.lastError()));

    if(formcursor.size()!=1) throw Error("Invalid Record!");
    formcursor.first();

    QString no=formcursor.value("no").toString();
    form.setRecord(formcursor.primeUpdate());
    form.insert(fileform->filename, "filename");
    //    form.insert(fileform->memo, "memo");
   
    form.insert(fileform->modified, "modified");
    form.insert(fileform->created, "created");
    form.readFields();

#ifdef HAS_KDE
    file="/tmp/test";
    QFile f(file);
    f.open(IO_WriteOnly | IO_Truncate);

    QSqlQuery q("select no from file where partdata_no="+no+" order by chunk_no");
    QProgressDialog progress( "Reading file...", "Abort reading", q.size(),
			      this, "progress", TRUE );

    int i=0;
    while(q.next()) {
      qApp->processEvents();
      QSqlQuery p("select chunk from file where no="+q.value(0).toString());
      p.first();
      QByteArray buf;
      //      debug(p.value(0).toCString());
      // if(!p.value(0).canCast(QVariant::ByteArray)) debug("Kann nicht nach QByteArray casten!");
      KCodecs::base64Decode(p.value(0).asCString(), buf);
      //      debug(QCString(buf));
      debug("Buf Size="+QString().setNum(buf.count()));
      QByteArray uncompr=qUncompress(buf);
      // uncompr.resize(uncompr.count()-2);
      // uncompr.resize(uncompr.count());
      f.writeBlock(uncompr);
      progress.setProgress(i);
      i++;
      QFile d("/tmp/block"+QString().setNum(i));
      d.open(IO_WriteOnly);
      d.writeBlock(uncompr);
      d.close();

    }
    f.close();
    
    KMimeMagicResult *mime;
    
    if(url().protocol()=="lit") {
      try {
	mime=KMimeMagic::self()->findFileType(file);
      } catch(...){
	QMessageBox::critical(this, "Error", "File does not exist!");
	return false;
      }
    }
    else {
      if (KIO::NetAccess::download(url(), kio_target) == false) {
	QMessageBox::critical(this, "Error", "File does not exist!");
	return false;
      }
      mime=KMimeMagic::self()->findFileType(kio_target);
      file=kio_target;
    }
    
    KTrader::OfferList offers = KTrader::self()->query(mime->mimeType(), "'KParts/ReadOnlyPart' in ServiceTypes");
    if(offers.count()>0) {
      KService::Ptr ptr = offers.first();
      if(ptr) {
	KLibFactory *factory = KLibLoader::self()->factory(ptr->library());
	
	if (factory) {
	  QValueList<int> list = fileform->splitter->sizes();
	  delete part; part=0L;
	  part = (KParts::ReadOnlyPart *)factory->create(fileform->splitter, ptr->name(), "KParts::ReadOnlyPart");
	  fileform->splitter->setSizes(list);
	  part->openURL(file);
	  part->widget()->show();
	  
	}
      }
    }
#endif

    formcursor.remove(formcursor.position("astext"));

    
    fileform->links->clear();
    filllink=new FillLinkListBox(this, "linkfilefllb", fileform->links, "select note.no, note.title from note, link where"+buildLinkWhereClause("note", "partdata", no)+" order by note.title","note/note", loadPixmap("shownote.png"));
    filllink->start();
    threads.append(filllink);

    setValid(true);
  } catch(Error &e) {
    setValid(false);
    setReadonlyGui(true);
    QMessageBox::warning(this, "Error", e.message());
    return false;
  }
  setValid(true);
  return true;
}

void File::setWorkingGui(bool work) {
  ReadWriteFormBase::setWorkingGui(work);
  fileform->links->setEnabled(!work);
}

void File::setReadonlyGui(bool ro) {
  debug("File::setReadonlyGui");
  ReadWriteFormBase::setReadonlyGui(ro);    
  fileform->filename->setEnabled(!ro);
}

void File::setInvalidGui(bool inv) {
  ReadWriteFormBase::setInvalidGui(inv);
  setWorkingGui(false);
  setReadonlyGui(inv);
}

void File::insertNote() {
  SelectNote *s=new SelectNote(this, "selnote");
  Q_ASSERT(s);
  if(s->exec()==QDialog::Accepted)
    createLink(s->url(), url());
}

void File::deleteNote() {
  deleteLink(fileform->links->url(), url());
}

void File::createNote() {
  newNote(url());
}

void File::addToProject() {
  SelectItem *s=new SelectItem(this, "selproject");
  Q_ASSERT(s);
  s->setData("project/project", "select no, title from project order by title");
  if(s->exec()==QDialog::Accepted) {
    QSqlQuery q("insert into project_bibliography (project_no, tab_no, id_no) values ("+
		no(s->url())+",tabid('partdata'), "+no(url())+")");
  }
}

#ifdef HAS_KDE
void File::runApp() {
  RunDialog r(this);
  r.setData("file://"+file);
  r.exec();
}
#endif
