/***************************************************************************
 *                                                                         *
 *   filllinklistbox.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "filllinklistbox.h"
#include "linklistbox.h"
#include <qthread.h>
#include <qapplication.h>
#include "linklistboxitem.h"
#include <qsqlcursor.h>
#include <qsqldatabase.h>
#include "dbdata.h"
#include "error.h"
#include <qmessagebox.h>
#include "threadfinishedevent.h"
#include "threadlinkevent.h"

QMutex filllinklistboxmutex;

FillLinkListBox::FillLinkListBox(QObject *parent, QString name, LinkListBox *box, QString query, QString cmd, QPixmap pm, QListViewItem *parentitem):  parent(parent), name(name), box(box), query(query), cmd(cmd), pixm(pm), parentitem(parentitem) {   
}

FillLinkListBox::~FillLinkListBox() {
 }


void FillLinkListBox::run() {
  while(filllinklistboxmutex.locked()) {
    //    qDebug("FillLinkListBox::run(): Locked - "+query);
    msleep(10);
  }

  QMutexLocker locker(&filllinklistboxmutex);
  //  qDebug("FillLinkListBox::run(): running - "+query);
  
  try {
        
    QSqlDatabase *conn=QSqlDatabase::database("threadConnection");
    QSqlQuery q(query, conn);
    if(!q.isActive()) 
      throw(Error(conn->lastError()));
        
    QSqlRecord r=conn->record(q);
    Q_ASSERT(r.position("no")!=-1);
    while(q.next()) {
      QString str;
      for(unsigned int i=0; i<r.count(); i++) 
	if(r.fieldName(i)!="no")	 
	  str+=q.value(i).toString()+" ";
            
      QApplication::postEvent(parent, new ThreadLinkEvent(box, str, LitUrl("lit://"+cmd+"?no="+q.value(r.position("no")).toString()), pixm, parentitem));
      
    }
    
  } catch(Error e) {
    qApp->lock();
    QMessageBox::warning(0L, "Error", e.message());
    qApp->unlock();
  }
  //  qDebug("FillLinkListBox::run(): return - "+query);  
}
