/***************************************************************************
 *                                                                         *
 *   fillsearchlist.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include <fillsearchlist.h>
#include <qregexp.h>
#include <qsqlquery.h>
#include <qsqldatabase.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include "threadsearchevent.h"
#include "global.h"
#include "error.h"


QString FillSearchList::buildResultRec(QString target, QString s, QStringList keywords, QString heading){
  QString res; 
  debug("buildResultRec() - begin");
  for(QStringList::Iterator it = keywords.begin(); it != keywords.end(); ++it ) {

    QString r;
    int pos=s.find(*it, 0, false);
    if(pos>-1) {
      r=s.mid( (pos-150>0?pos-150:0), 300+QString(*it).length() );
      r="..."+r+"...";
      res+=r.replace(QRegExp("("+*it+")", false), "<a href=\""+target+"\"><font color=red>\\1</font></a>").simplifyWhiteSpace();
      debug(res);
    }
  }
  res="<b>"+heading+"</b><br>"+res+"<br>";
  debug("buildResultRec() - ende");
  return res;
}

QMutex fillsearchlistmutex;
FillSearchList::FillSearchList(QObject *parent, QStringList terms): parent(parent), terms(terms) {
}

FillSearchList::~FillSearchList() {
}

void FillSearchList::run() {
  while(fillsearchlistmutex.locked()) 
    msleep(10);

  QMutexLocker locker(&fillsearchlistmutex);
  
  try {
    if(!createConnection("qt_search_connection")) throw(Error("Cannot open database connection"));
    QSqlDatabase *conn=QSqlDatabase::database("qt_search_connection");
    QStringList sl=terms;
    QString q;

    for(QStringList::Iterator it = sl.begin(); it != sl.end(); ++it)
      q+="select distinct * from search_term('"+QString(*it)+"') as (a text, b text) intersect ";

    QSqlQuery query(q.left(q.length()-11), conn);
    if(!query.isActive()) 
      throw(Error(query.lastError()));

    QPtrList<QString> ids;
    ids.setAutoDelete(true);

    while(query.next()) 
      ids.append(new QString("(XXTABLEXX.no="+query.value(0).toString()+" and tabid('XXTABLEXX')="+query.value(1).toString()+") or "));

    //    if(searchform->fulltext->isChecked()) {
    addResult("select publication.title as ordertitle, part.title, partdata.no, partdata.astext from publication, part, partdata where publication.no=part.publication_no and part.no = partdata.part_no ", "partdata", "file/partdata", ids, sl);

    addResult("select publication.title as ordertitle, part.title, part.no, part.memo from publication, part where publication.no=part.publication_no ", "part", "part/part", ids, sl);

    addResult("select note.title as ordertitle, '', note.no, note.memo from note where true", "note", "note/note", ids, sl);

    addResult("select publication.title as ordertitle, '', publication.no, publication.memo from publication where true", "publication", "publication/publication", ids, sl);

    addResult("select author.firstname, author.lastname as ordertitle, author.no, author.memo from author where true", "author", "author/author", ids, sl);

    //    }
  } catch(Error &e) {
    qApp->lock();
    QMessageBox::warning(0L, "Error", e.message());
    qApp->unlock();
  }
}


void FillSearchList::addResult(QString where, QString table, QString cmd, QPtrList<QString> ids, QStringList sl) {
  if(ids.count()>0) {
    QString q=where+" and (";
    QString *t;
    for(t=ids.first(); t; t=ids.next()) 
      q+=QString(*t).replace("XXTABLEXX", table);
    QSqlQuery query(q.left(q.length()-4)+") order by ordertitle");
    
    if(!query.isActive()) 
      throw(Error(query.lastError()));
    
    while(query.next()) {
      QString heading=query.value(0).toString()+" - "+query.value(1).toString();
      QString res=buildResultRec("lit://"+cmd+"?no="+query.value(2).toString(), query.value(3).toString(), sl, heading);
      debug(res);
      QApplication::postEvent(parent, new ThreadSearchEvent(res));
    }    
  }
}

