/***************************************************************************
 *                                                                         *
 *   linklistbox.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "linklistbox.h"
#include "linklistboxitem.h"
#include <qcursor.h>
#include <qapplication.h>
#include "liturl.h"
#include "urlevent.h"

LinkListBox::LinkListBox(QWidget *parent, const char *name ) :
		QListView(parent,name) {
  setFocusPolicy( QWidget::StrongFocus );
  setBackgroundMode( QWidget::PaletteBase );
  addColumn("");
  
  pop.insertItem("Open", this, SLOT(slotOpen()));
  pop.insertItem("Open in new Window", this, SLOT(slotNewWindow()));
  
  connect(this, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint &,int)),
	  this, SLOT(slotPopupMenu(QListViewItem *, const QPoint &,int)));

  connect(this, SIGNAL(doubleClicked(QListViewItem*)),
	  this, SLOT(slotSelected(QListViewItem*)));
  
}

void LinkListBox::open(bool newwin) {
    LinkListBoxItem *i=dynamic_cast<LinkListBoxItem*>(currentItem());
    ASSERT(i);
    qApp->postEvent(topLevelWidget(), new UrlEvent(i->url(), newwin));
}

void LinkListBox::slotSelected(QListViewItem *item) {
  open(false);
}

void LinkListBox::slotOpen() {
  open(false);
}


void LinkListBox::slotNewWindow() {
 open(true);
}


void LinkListBox::slotPopupMenu(QListViewItem *item, const QPoint& p, int c) {
	if(item) {
  	  setCurrentItem(item);
  	  pop.popup(QCursor::pos());
    }
}

LinkListBox::~LinkListBox(){
}

LitUrl LinkListBox::url() {
    LinkListBoxItem *i=dynamic_cast<LinkListBoxItem *>(currentItem());
    if(i)
      return i->url();
    else
      return LitUrl();
}
