/***************************************************************************
 *                                                                         *
 *   linklistboxitem.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "linklistboxitem.h"
#include <qlistview.h>
#include "liturl.h"
#include <qcolor.h>
#include <qpalette.h>

LinkListBoxItem::LinkListBoxItem(QListView *listbox, QString title, LitUrl url, QPixmap pm):
  QListViewItem(listbox, title), u(url) {
  setPixmap(0, pm);      
  init();
}

LinkListBoxItem::LinkListBoxItem(QListView *listbox, LitUrl url, QListViewItem *after,  QPixmap pm):
  QListViewItem(listbox, after), u(url) {
  setPixmap(0, pm);      
  init();
}

LinkListBoxItem::LinkListBoxItem(QListViewItem *parent, QString title, LitUrl url, QPixmap pm):
  QListViewItem(parent, title), u(url) {
  setPixmap(0, pm);      
  init();
}

LinkListBoxItem::LinkListBoxItem(QListViewItem *parent, LitUrl url, QListViewItem *after,  QPixmap pm):
  QListViewItem(parent, after), u(url) {
  setPixmap(0, pm);      
  init();
}

void LinkListBoxItem::init() {
  backgrcolor=Qt::white;
  textcolor=Qt::black;
  orderprefix="m";
}

void LinkListBoxItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int alignment) {
  QColorGroup mycg(cg);
  mycg.setColor(QColorGroup::Text, textcolor);
  mycg.setColor(QColorGroup::Base, backgrcolor);	
  QListViewItem::paintCell(p, mycg, column, width, alignment);
}

LitUrl LinkListBoxItem::url() { return u; };
  
void LinkListBoxItem::setTextColor(const QColor& c) {
  textcolor=c;
}
  
void LinkListBoxItem::setBackgroundColor(const QColor& c) {
  backgrcolor=c;
}
	
void LinkListBoxItem::setOrderPrefix(QString p) {
  orderprefix=p;
}

const QString LinkListBoxItem::key(int column, bool ascending) {
  debug("key");
  return orderprefix+text(column);
}
    
LinkListBoxItem::~LinkListBoxItem() {
}
