/***************************************************************************
 *                                                                         *
 *   mainwindowimpl.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "mainwindowimpl.h"
#include <qapplication.h>
#include <qlineedit.h>
#include <qaction.h>
#include <qevent.h>
#include "global.h"
#include "urlevent.h"
#include "author.h"
#include "publication.h"
#include "bibliography.h"
#include "authoroverview.h"
#include "publoverview.h"
#include "noteoverview.h"
#include "journaloverview.h"
#include "publisheroverview.h"
#include "projectoverview.h"
#include "project.h"
#include "part.h"
#include "partmemo.h"
#include "note.h"
#include "file.h"
#include "createhtml.h"
#include "state.h"
#include "search.h"
#include "journal.h"
#include "publisher.h"
#include <qsettings.h>

MainWindowImpl::MainWindowImpl(QWidget *parent, const char *name): MainWindow(parent, name), widget(0L) {
  backAction->setEnabled(false);    
  forwardAction->setEnabled(false);


  QSettings s;
  s.setPath("www.test.de", "Literature");
  //  s.writeEntry("/literature/mimetotext/application/pdf", "t=/tmp/`date +%H%M%S`$RANDOM; pdftohtml -iFd $t -o - #in# | html2text -o #out#; rm -r $t");
  s.writeEntry("/literature/mimetotext/application/pdf", "pdftotext #in# #out#");
  s.writeEntry("/literature/mimetotext/text/html", "cat #in# | /usr/bin/html2text -o #out#");
  s.writeEntry("/literature/mimetotext/text/plain", "#direct#");
  bool ok;
  QString b=s.readEntry("/literature/mime2text/application/pdf", QString::null, &ok);
  debug("Befehl ist: "+b);


}

MainWindowImpl::~MainWindowImpl() {
}

void MainWindowImpl::closeEvent(QCloseEvent *e) {
  if(widget)
    widget->close(true);
  e->accept();
}

void MainWindowImpl::setupBackForward() {
  if(widget && widget->isValid() &&
     !(stforward.top() && stforward.top()->url()==widget->url()) &&
     !(stback.top() && stback.top()->url()==widget->url())) {
    stback.push(new UrlEvent(widget->url(), false));
    backAction->setEnabled(true);
    stforward.clear();
    forwardAction->setEnabled(false);
  }
}

void MainWindowImpl::customEvent(QCustomEvent *e) {
  UrlEvent *u=dynamic_cast<UrlEvent*>(e);    
  if(u) {    
    LitUrl url=u->url();
    debug("MainWindowImpl::customEvent"+QString(url));
    if(u->newWin()) {
      MainWindowImpl* new_window= new MainWindowImpl();
      u->setNewWin(false);
      qApp->postEvent(new_window, new UrlEvent(*u));
      new_window->show();
    } else {
      setupBackForward();
      urlline->setText(url);
      if(widget) widget->close(true);
      widget=0L;
      if(url.form()=="author") showAuthor(url);
      else if(url.form()=="publication") showPublication(url);
      else if(url.form()=="part") showPart(url);	    
      else if(url.form()=="partmemo") showPartMemo(url);
      else if(url.form()=="note") showNote(url);	    
      else if(url.form()=="file") showFile(url);	    	    
      else if(url.form()=="bibliography") showBibliography(url);	    
      else if(url.form()=="state") showState(url);
      else if(url.form()=="search") showSearch(url);      
      else if(url.form()=="authoroverview") showAuthorOverview(url);
      else if(url.form()=="publicationoverview") showPublOverview(url);
      else if(url.form()=="noteoverview") showNoteOverview(url);
      else if(url.form()=="journaloverview") showJournalOverview(url); 
      else if(url.form()=="publisheroverview") showPublisherOverview(url);       
      else if(url.form()=="publisher") showPublisher(url);     
      else if(url.form()=="projectoverview") showProjectOverview(url);       
      else if(url.form()=="project") showProject(url);     
      else if(url.form()=="journal") showJournal(url);           
      else showState(url);
    }
  }
}

void MainWindowImpl::setupWidget(QWidget *widget) {
  widget->setFocus();
  setCentralWidget(widget);
  widget->show();
}

void MainWindowImpl::showNote( LitUrl url )
{
  widget=new Note(url, this, SLOT(setWorkingGui(bool)), this,"noteframe");
  setupWidget(widget);
}

void MainWindowImpl::showAuthor( LitUrl url )
{
  widget=new Author(url, this, SLOT(setWorkingGui(bool)), this,"authorframe");
  setupWidget(widget);
}

void MainWindowImpl::showPublication( LitUrl url )
{
  widget=new Publication(url, this, SLOT(setWorkingGui(bool)), this,"publframe");
  setupWidget(widget);
}

void MainWindowImpl::showPart( LitUrl url )
{
  widget=new Part(url, this, SLOT(setWorkingGui(bool)), this,"partframe");
  setupWidget(widget);
}

void MainWindowImpl::showPartMemo( LitUrl url )
{
  widget=new PartMemo(url, this, SLOT(setWorkingGui(bool)), this,"partmemoframe");
  setupWidget(widget);
}


void MainWindowImpl::showBibliography( LitUrl url )
{
  widget=new Bibliography(url, this, SLOT(setWorkingGui(bool)), this,"biblioframe");
  setupWidget(widget);
}


void MainWindowImpl::showFile( LitUrl url )
{
  widget=new File(url, this, SLOT(setWorkingGui(bool)), this,"fileframe");
  setupWidget(widget);
}

void MainWindowImpl::showAuthorOverview( LitUrl url )
{
  widget=new  AuthorOverview(url, this, SLOT(setWorkingGui(bool)), this, "authoroverview");
  setupWidget(widget);
}

void MainWindowImpl::showNoteOverview( LitUrl url )
{
  widget=new  NoteOverview(url, this, SLOT(setWorkingGui(bool)),  this, "noteoverview");
  setupWidget(widget);
}

void MainWindowImpl::showPublOverview( LitUrl url )
{
  widget=new PublOverview(url, this, SLOT(setWorkingGui(bool)), this, "publoverview");
  setupWidget(widget);
}

void MainWindowImpl::showJournalOverview( LitUrl url )
{
  widget=new JournalOverview(url, this, SLOT(setWorkingGui(bool)), this, "publoverview");
  setupWidget(widget);
}

void MainWindowImpl::showPublisherOverview( LitUrl url )
{
  widget=new PublisherOverview(url, this, SLOT(setWorkingGui(bool)), this, "publoverview");
  setupWidget(widget);
}

void MainWindowImpl::showState(LitUrl url)
{
  widget=new State(url, this, SLOT(setWorkingGui(bool)), this, "state");
  setupWidget(widget);
}

void MainWindowImpl::showSearch(LitUrl url)
{
  widget=new Search(url, this, SLOT(setWorkingGui(bool)), this, "search");
  setupWidget(widget);
}

void MainWindowImpl::showJournal(LitUrl url)
{
  widget=new Journal(url, this, SLOT(setWorkingGui(bool)), this, "journal");
  setupWidget(widget);
}

void MainWindowImpl::showPublisher(LitUrl url)
{
  widget=new Publisher(url, this, SLOT(setWorkingGui(bool)), this, "publisher");
  setupWidget(widget);
}

void MainWindowImpl::showProject(LitUrl url)
{
  widget=new Project(url, this, SLOT(setWorkingGui(bool)), this, "project");
  setupWidget(widget);
}

void MainWindowImpl::showProjectOverview(LitUrl url)
{
  widget=new ProjectOverview(url, this, SLOT(setWorkingGui(bool)), this, "projectoverview");
  setupWidget(widget);
}


void MainWindowImpl::forward()
{
  qDebug("forward");
  if(stforward.isEmpty())
    return;
  
  if(widget && widget->isValid()) {
    stback.push(new UrlEvent(widget->url(), false));
    if(!stback.isEmpty()) backAction->setEnabled(true);
  }
  
  qApp->postEvent(this, stforward.pop());   
  if(stforward.isEmpty())
    forwardAction->setEnabled(false);
}

void MainWindowImpl::back()
{
  if(stback.isEmpty())
    return;
  if(widget && widget->isValid()) {
    stforward.push(new UrlEvent(widget->url(), false));
    if(!stforward.isEmpty())
      forwardAction->setEnabled(true);
  }
  qApp->postEvent(this, stback.pop());       
  if(stback.isEmpty())
    backAction->setEnabled(false);
}
