/***************************************************************************
 *                                                                         *
 *   mainwindowimpl.h								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "mainwindow.h"


class QLineEdit;
class UrlEvent;
class QCloseEvent;
class ReadOnlyFormBase;
class LitUrl;

/** 
    @short Displays the dialogs, handels back/forward and global menus.
  
     Loading and unloading of the dialogs is handelt via events and urls. 
     A custom event "UrlEvent" holds the information. Its processed in 
     "customEvent()". If received, the method closes the old dialig and 
     calls the appropriate method to load the new dialog (showAuthor(), 
     showNoteOverview(), ...) and initializes the back/forward mechanism.
     
     Each Instance has its own database connection. It can be accessed via
     "dbConn()".
    */
class MainWindowImpl: public MainWindow {
   Q_OBJECT

private:
    MainWindowImpl(const MainWindowImpl&);
    MainWindowImpl& operator=(const MainWindowImpl&);
    void setupBackForward(); 
    ReadOnlyFormBase *widget;
    void closeEvent(QCloseEvent *e);

public:
    MainWindowImpl(QWidget *parent=0, const char *name=0);
    void setupWidget(QWidget *widget);
    void showAuthor(LitUrl url);
    void showPublication(LitUrl url);
    void showBibliography( LitUrl url );
    void showAuthorOverview(LitUrl url);
    void showNoteOverview(LitUrl url);
    void showPublOverview(LitUrl url);
    void showPublisherOverview(LitUrl url);    
    void showJournalOverview(LitUrl url);    
    void showProjectOverview(LitUrl url);        	
    void showState(LitUrl url);
    void showSearch(LitUrl url);    
    void showPart(LitUrl url);
    void showPartMemo(LitUrl url);
    void showFile(LitUrl url);    
    void showNote(LitUrl url);
    void showJournal(LitUrl url);
    void showPublisher(LitUrl url);    
    void showProject(LitUrl url);        
    /** UrlEvent processing. Handles creation and deletion of dialogs.
      */
    void customEvent(QCustomEvent *e);
    ~MainWindowImpl();
    void back();
    void forward();
};
