/***************************************************************************
 *                                                                         *
 *   note.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "note.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qcombobox.h>
#include <qsettings.h>
#include <qpixmap.h>
#include "error.h"
#include "editorhelper.h"
#include "global.h"
#include "linklistbox.h"
#include "linklistboxitem.h"
#include "filllinklistbox.h"
#include "mainwindowimpl.h"
#include <qlayout.h>
#include <selectnote.h>

Note::Note(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name), toolbar(0L) {    
  noteform=new NoteForm(this, "noteform");
  QHBoxLayout *l=new QHBoxLayout(this);
  l->addWidget(noteform);
  QSettings settings;
  QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
  if(!backgr.isEmpty()) noteform->setErasePixmap(QPixmap(backgr));
  initToolBar();
  new EditorHelper(this, noteform->memo);
  setData();
}

void Note::initToolBar() {
    toolbar=new QToolBar(mainWindow());
    
    insertNoteAction=new QAction("Insert Note", QIconSet(loadPixmap("insnote.png")), "Insert Note", 0, this, "insertnote");
    connect(insertNoteAction, SIGNAL(activated()), this, SLOT(insertNote()));
    deleteNoteAction=new QAction("Delete Note", QIconSet(loadPixmap("delnote.png")), "Delete Note", 0, this, "deletenote");
    connect(deleteNoteAction, SIGNAL(activated()), this, SLOT(deleteNote()));
    
    insertNoteAction->addTo(toolbar);
    deleteNoteAction->addTo(toolbar);
    toolbar->show();

    insertNoteAction->addTo(roMenu());
    deleteNoteAction->addTo(roMenu());

}

Note::~Note() {
  write();
  delete toolbar;
}

QListViewItem *Note::addNoteTree(QString where, QListViewItem *par, bool emph) {
  QSqlQuery q("select title, no from note where "+where);
  LinkListBoxItem *parentitem=0L;
  while(q.next()) {
    QString s=q.value(0).toString();
    LitUrl u("lit://note/note?no="+q.value(1).toString());
    if(par) parentitem = new LinkListBoxItem(par, s, u, loadPixmap("shownote.png"));			else parentitem = new LinkListBoxItem(noteform->links, s, u, loadPixmap("shownote.png"));
    parentitem->setTextColor(Qt::black);
    if(emph) parentitem->setBackgroundColor(Qt::blue);
    else parentitem->setBackgroundColor(Qt::lightGray);
    parentitem->setOrderPrefix("b");
    parentitem->setOpen(true);
  }
  return parentitem;
}

bool Note::setData()
{
  try {
    ReadWriteFormBase::setData();
    LitUrl u=url();
    setWorkingGui(true);
    form.clear();
    formcursor.clear();
    formcursor.setMode(QSqlCursor::Writable);
    formcursor.setName("note");
    if(!formcursor.select(u.query()))
      throw(Error(formcursor.lastError()));
    qDebug("count:"+QString().setNum(formcursor.size()));
    if(formcursor.size()!=1) throw Error("Invalid Record!");
    formcursor.first();
    QString no=formcursor.value("no").toString();
    form.setRecord(formcursor.primeUpdate());
    form.insert(noteform->title, "title");
    form.insert(noteform->memo, "memo");
    form.insert(noteform->modified, "modified");
    form.insert(noteform->created, "created");
    form.readFields();

    noteform->links->setEnabled(false);
    noteform->links->clear();	
    int type=formcursor.value("type").toInt();
    QString parent=formcursor.value("parent").toString();
	
    QListViewItem *parentitem=0L;
    QSqlQuery q;
    if(type==0) {
      noteform->links->setRootIsDecorated(true);	    
      if(parent!="0") parentitem=addNoteTree("no="+parent);
      parentitem=addNoteTree("no="+no, parentitem, true);
      parentitem=addNoteTree("parent="+no,parentitem);
    }
	    
    filllink_note=new FillLinkListBox(this, "linknotefllb", noteform->links, "select note.no, note.title from note, link where "+buildLinkWhereClause("note", "note", no), "note/note", loadPixmap("shownote.png"));
    filllink_note->start();
    threads.append(filllink_note);
	
				      filllink_author=new FillLinkListBox(this, "linkauthorfllb", noteform->links, "select author.no, author.firstname, author.lastname from author, link where"+buildLinkWhereClause("author", "note", no), "author/author", loadPixmap("showauthor.png"));
    filllink_author->start();
    threads.append(filllink_author);
	
    if(type==1) {
      filllink_publ=new FillLinkListBox(this, "linkpublfllb", noteform->links, "select publication.no, publication.title from publication, link where "+buildLinkWhereClause("publication", "note", no), "publication/publication", loadPixmap("showpubl.png"));
      filllink_publ->start();
      threads.append(filllink_publ);
	
      filllink_part=new FillLinkListBox(this, "linkpartfllb", noteform->links, "select part.no, part.title from part, link where"+buildLinkWhereClause("part", "note", no), "part/part", loadPixmap("showpart.png"));
      filllink_part->start();
      threads.append(filllink_part);

      filllink_partdata=new FillLinkListBox(this, "linkpartdatafllb", noteform->links, "select partdata.no, partdata.filename from partdata, link where"+buildLinkWhereClause("partdata", "note", no), "file/partdata", loadPixmap("showfile.png"));
      filllink_partdata->start();
      threads.append(filllink_partdata);
    }
	
    setValid(true);
  } catch(Error &e) {
    setValid(false);
    setReadonlyGui(true);
    QMessageBox::warning(this, "Error", e.message());
    return false;
  }
  setValid(true);
  return true;
}

void Note::setWorkingGui(bool work) {
  ReadWriteFormBase::setWorkingGui(work);    
  noteform->links->setEnabled(!work);
}

void Note::setReadonlyGui(bool ro) {
  ReadWriteFormBase::setReadonlyGui(ro);    
  noteform->title->setEnabled(!ro);
  noteform->memo->setEnabled(!ro);
}

void Note::setInvalidGui(bool inv) {
  ReadWriteFormBase::setInvalidGui(inv);    
  setWorkingGui(false);
  setReadonlyGui(inv);
}

void Note::deleteRec()
{
  if(!isValid()) return;
  QSqlQuery query( "select count(*) from note where parent="+no(url()));
  query.first();
  if(query.value(0).toInt()>0)
    QMessageBox::warning(this, "Error", "Can't delete note since one or more notes are child notes of this note. Please delete them first.");
  else 
    ReadWriteFormBase::deleteRec();
}


void Note::insertNote() {
  SelectNote *s=new SelectNote(this, "selnote");
  Q_ASSERT(s);
  if(s->exec()==QDialog::Accepted)
    createLink(s->url(), url());
}

void Note::deleteNote() {
  deleteLink(noteform->links->url(), url());
}
