/***************************************************************************
 *                                                                         *
 *   noteoverviewform.ui.h								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/
struct noterec {
    int     no;
    QString title;
    int     type;
    int     publication_no;
    int     parent;
    int     sibling;
};

void NoteOverviewForm::init() {
    setFocusPolicy( QWidget::StrongFocus );
    setBackgroundMode( QWidget::PaletteBase );
    notesview->addColumn( "Notes" );
    notesview-> setRootIsDecorated(true);
    notesview->setAcceptDrops(true);
    pop.insertItem("Open", this, SLOT(slotOpen()));
    pop.insertItem("Open in new Window", this, SLOT(slotNewWindow()));
    pop.insertItem("New Item", this, SLOT(slotNewItem()));  
    
    connect(notesview, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &,int)),
	    this, SLOT(slotPopupMenu(QListViewItem *, const QPoint &,int)));
    
    connect(notesview, SIGNAL(doubleClicked(QListViewItem *)),
	    this, SLOT(slotSelected(QListViewItem*)));
    
    connect(notesview, SIGNAL(returnPressed(QListViewItem *)),
	    this, SLOT(slotSelected(QListViewItem*)));
}

void NoteOverviewForm::setData(bool withpubls)
{
    notesview->clear();
    QPtrList<noterec> n;
    QSqlQuery query("select no, title, type, publication_no, parent, sibling from note where type=0 order by title");
    while(query.next()) {
	noterec *r=new noterec;
	r->no=query.value(0).toInt();
	r->title=query.value(1).toString();
	r->type=query.value(2).toInt();
	r->publication_no=query.value(3).toInt();
	r->parent=query.value(4).toInt();
	r->sibling=query.value(5).toInt();
	n.append(r);
    }
    
    if(withpubls) {
	query.exec("select no into temporary table tmp_notetree from note where type=0");
	query.exec("select note.no, note.title, note.type, note.publication_no, link.obj_1 as parent, sibling from note, link, tmp_notetree where link.obj_2=note.no and link.obj_1 = tmp_notetree.no and note.type=1 union select note.no, note.title, note.type, note.publication_no, link.obj_2 as parent, sibling from note, link, tmp_notetree where link.obj_1=note.no and link.obj_2 = tmp_notetree.no and note.type=1");
	while(query.next()) {
	    noterec *r=new noterec;
	    r->no=query.value(0).toInt();
	    r->title=query.value(1).toString();
	    r->type=query.value(2).toInt();
	    r->publication_no=query.value(3).toInt();
	    r->parent=query.value(4).toInt();
	    r->sibling=query.value(5).toInt();
	    n.append(r);
	}
	query.exec("drop table tmp_notetree");
    }
    
    addItem(n, 0, 0L);
    notesview->geometry();
}

void NoteOverviewForm::addItem(QPtrList<noterec>& n, int parentoid, NoteTreeItem *parentitem) {
    QPtrListIterator<noterec> it(n);
    noterec *rec;
    while ((rec = it.current()) != 0 ) {
	++it;
	if(rec->parent==parentoid) {
	    NoteTreeItem *i;
	    if(parentitem)
		i=new NoteTreeItem(parentitem, LitUrl("lit://note/note?no="+QString().number(rec->no)), rec->title, loadPixmap(rec->type==1?"showpubl.png":"shownote.png"), rec->parent, rec->sibling);
	    else
		i=new NoteTreeItem(notesview, LitUrl("lit://note/note?no="+QString().number(rec->no)), rec->title, loadPixmap(rec->type==1?"showpubl.png":"shownote.png"), rec->parent, rec->sibling);
  	    i->setDragEnabled(rec->type==0);
	    addItem(n, rec->no, i);
	    n.remove(rec);
	}
    }
}


LitUrl NoteOverviewForm::url() {
    if(!notesview->currentItem()) return LitUrl();
    NoteTreeItem *i=dynamic_cast<NoteTreeItem*>(notesview->currentItem());
    Q_ASSERT(i);
    return i->url();
}


void NoteOverviewForm::open(bool newwin) {
    NoteTreeItem *i=dynamic_cast<NoteTreeItem*>(notesview->currentItem());
    Q_ASSERT(i);
    qApp->postEvent(topLevelWidget(), new UrlEvent(i->url(), newwin));
}

void NoteOverviewForm::slotSelected(QListViewItem *i) {
    open(false);
}

void NoteOverviewForm::slotOpen() {
    open(false);
}

void NoteOverviewForm::slotNewWindow() {
    open(true);
}

void NoteOverviewForm::slotNewItem() {
    NoteTreeItem *i=dynamic_cast<NoteTreeItem*>(notesview->currentItem());
    Q_ASSERT(i);
    QSqlQuery q("select create_toplevel_note(no) from "+i->url().fileName()+" where "+i->url().query());
   q.first();
   qApp->postEvent(topLevelWidget(), new UrlEvent(LitUrl("lit://note/note?no="+q.value(0).toString()), false));
}


void NoteOverviewForm::slotPopupMenu(QListViewItem *item, const QPoint& p, int c) {
    notesview->setCurrentItem(item);
    pop.popup(QCursor::pos());
}
