/***************************************************************************
 *                                                                         *
 *   part.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "part.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qcombobox.h>
#include <qsettings.h>
#include "error.h"
#include "editorhelper.h"
#include "global.h"
#include "linklistbox.h"
#include "authorlinklistbox.h"
#include "filllinklistbox.h"
#include "mainwindowimpl.h"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qprogressdialog.h>
#include "urlevent.h"
#include "bibtexfactory.h"
#include "selectnote.h"
#include "selectitem.h"
#include "convert2text.h"
#include <importprogress.h>



#ifdef HAS_KDE
#include <kmimemagic.h>
#include <qcstring.h>
#include <kmdcodec.h>
#endif

Part::Part(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name), toolbar(0L) {    
  partform=new PartForm(this, "partform");
  QHBoxLayout *l=new QHBoxLayout(this);
  l->addWidget(partform);
    
  QSettings settings;
  settings.setPath("www.test.de", "Literature");
  QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
  if(!backgr.isEmpty()) partform->setErasePixmap(QPixmap(backgr));
  connect(partform->genbibtex, SIGNAL(clicked()), this, SLOT(generateBibTex()));

  initToolBar();
  setData();
}

void Part::initToolBar() {
  toolbar=new QToolBar(mainWindow());

  showPartMemoAction=new QAction("Show Memo", QIconSet(loadPixmap("showmemo.png")), "Show Memo", 0, this, "showmemo");
  connect(showPartMemoAction, SIGNAL(activated()), this, SLOT(showPartMemo()));

  showPartPublAction=new QAction("Show Publication", QIconSet(loadPixmap("showpubl.png")), "Show Publication", 0, this, "showpartpubl");
  connect(showPartPublAction, SIGNAL(activated()), this, SLOT(showPartPubl()));
   
  // **** Note ****
  insertNoteAction=new QAction("Insert Note", QIconSet(loadPixmap("insnote.png")), "Insert Note", 0, this, "insertnote");
  connect(insertNoteAction, SIGNAL(activated()), this, SLOT(insertNote()));
    
  createNoteAction=new QAction("Create Note", QIconSet(loadPixmap("newnote.png")), "Create Note", 0, this, "createnote");
  connect(createNoteAction, SIGNAL(activated()), this, SLOT(createNote()));
  
  deleteNoteAction=new QAction("Delete Note", QIconSet(loadPixmap("delnote.png")), "Delete Note", 0, this, "deletenote");
  connect(deleteNoteAction, SIGNAL(activated()), this, SLOT(deleteNote()));
  
  // **** Author ****
  insertAuthorAction=new QAction("Insert Author", QIconSet(loadPixmap("insauthor.png")), "Insert Author", 0, this, "insertauthor");
  connect(insertAuthorAction, SIGNAL(activated()), this, SLOT(insertAuthor()));
  createAuthorAction=new QAction("Create Author", QIconSet(loadPixmap("newauthor.png")), "Create Author", 0, this, "createauthor");
  connect(createAuthorAction, SIGNAL(activated()), this, SLOT(createAuthor()));
  deleteAuthorAction=new QAction("Delete Author", QIconSet(loadPixmap("delauthor.png")), "Delete Author", 0, this, "deleteauthor");
  connect(deleteAuthorAction, SIGNAL(activated()), this, SLOT(deleteAuthor()));

  importFilesAction=new QAction("Import Files", QIconSet(loadPixmap("gnome-folder.png")), "Import Files", 0, this, "importfiles");
  connect(importFilesAction, SIGNAL(activated()), this, SLOT(importFiles()));

  addToProjectAction=new QAction("Add to Project", QIconSet(loadPixmap("gnome-folder.png")), "Add to Project", 0, this, "addtoproject");
  connect(addToProjectAction, SIGNAL(activated()), this, SLOT(addToProject()));


  showPartPublAction->addTo(toolbar);
  showPartMemoAction->addTo(toolbar);

  insertNoteAction->addTo(toolbar);
  createNoteAction->addTo(toolbar);
  deleteNoteAction->addTo(toolbar);

  insertAuthorAction->addTo(toolbar);
  createAuthorAction->addTo(toolbar);
  deleteAuthorAction->addTo(toolbar);

  importFilesAction->addTo(toolbar);
  addToProjectAction->addTo(toolbar);
   
  toolbar->show();

  showPartPublAction->addTo(roMenu());
  showPartMemoAction->addTo(roMenu());

  insertNoteAction->addTo(roMenu());
  createNoteAction->addTo(roMenu());
  deleteNoteAction->addTo(roMenu());

  insertAuthorAction->addTo(roMenu());
  createAuthorAction->addTo(roMenu());
  deleteAuthorAction->addTo(roMenu());

  importFilesAction->addTo(roMenu());
  addToProjectAction->addTo(roMenu());

}

Part::~Part() {
  write();
  delete toolbar;
}

bool Part::setData()
{
  try {
    ReadWriteFormBase::setData();
    LitUrl u=url();
    setWorkingGui(true);
    form.clear();
    formcursor.clear();
    formcursor.setMode(QSqlCursor::Writable);
    formcursor.setName("part");
    if(!formcursor.select(u.query()))
      throw(Error(formcursor.lastError()));
    if(formcursor.size()!=1) throw Error("Invalid Record!");
    formcursor.first();
    QString no=formcursor.value("no").toString();
    form.setRecord(formcursor.primeUpdate());
    form.insert(partform->number, "partno");	
    form.insert(partform->title, "title");
    form.insert(partform->key, "key");
    form.insert(partform->pages, "pages");
    form.insert(partform->bibtex, "bibtex");
    form.insert(partform->modified, "modified");
    form.insert(partform->created, "created");
    form.readFields();

    partform->links->setEnabled(false);
    partform->authors->setEnabled(false);
    partform->files->setEnabled(false);
	
    partform->authors->clear();
    fillauthor=new FillLinkListBox(this, "partauthorfllb", partform->authors, "select author.no, author.firstname, author.lastname from author, part_author where author.no=part_author.author_no and part_author.part_no="+no+" order by part_author.author_order desc", "author/author", loadPixmap("showauthor.png"));
    fillauthor->start();
    threads.append(fillauthor);

    partform->files->clear();
    fillfiles=new FillLinkListBox(this, "filesfllb", partform->files, "select partdata.no, partdata.filename from partdata where partdata.part_no="+no+" order by partdata.filename","file/partdata", loadPixmap("showfile.png"));
    fillfiles->start();
    threads.append(fillfiles);

    partform->links->clear();
    filllink=new FillLinkListBox(this, "linkpartfllb", partform->links, "select note.no, note.title from note, link where"+buildLinkWhereClause("note", "part", no), "note/note", loadPixmap("shownote.png"));
    filllink->start();
    threads.append(filllink);
	
    setValid(true);
  } catch(Error &e) {
    setValid(false);
    setReadonlyGui(true);
    QMessageBox::warning(this, "Error", e.message());
    return false;
  }
  setValid(true);
  return true;
}

void Part::write() {
  ReadWriteFormBase::write();
  if(!partform->authors->modified()) return;
  typedef QValueList<LitUrl> List;
  List l=partform->authors->authorPos();
  
  List::iterator it;
  int i=0;
  for ( it = l.begin(); it != l.end(); ++it ) {
    QString n=no(*it);
    QSqlQuery("update part_author set author_order="+QString().setNum(i)+" where part_no="+no(url())+" and author_no="+n);
    i++;
  }
}


void Part::setWorkingGui(bool work) {
  ReadWriteFormBase::setInvalidGui(work);
  partform->links->setEnabled(!work);
  partform->authors->setEnabled(!work);
  partform->files->setEnabled(!work);
}

void Part::setReadonlyGui(bool ro) {
  ReadWriteFormBase::setReadonlyGui(ro);
  partform->title->setEnabled(!ro);
  partform->bibtex->setEnabled(!ro);
  partform->key->setEnabled(!ro);
  partform->number->setEnabled(!ro);
  partform->pages->setEnabled(!ro);    
}

void Part::setInvalidGui(bool inv) {
  ReadWriteFormBase::setInvalidGui(inv);      
  setWorkingGui(false);
  setReadonlyGui(inv);
}

void Part::showPartMemo() {
  qApp->postEvent(topLevelWidget(), new UrlEvent(LitUrl("lit://partmemo/part?no="+no(url())), false));
}

void Part::generateBibTex() {
  // FixMe
  //  write();
  //  setData();
  debug("Part::generateBibTex()");
  PartBibTexFactory f(formcursor);
  debug(f.getEntry());
  partform->bibtex->setText(f.getEntry());
}


void Part::insertNote() {
  SelectItem *s=new SelectItem(this, "selnote");
  Q_ASSERT(s);
  s->setData("note/note", "select note.no, note.title from note, part where note.publication_no=part.publication_no and part.no="+no(url())+" order by note.title");
  if(s->exec()==QDialog::Accepted)
    createLink(s->url(), url());
}

void Part::deleteNote() {
  deleteLink(partform->links->url(), url());    
}

void Part::createNote() {
  QSqlQuery q("select publication_no from part where no="+no(url()));
  q.first();
  newNote(LitUrl("lit://publication/publication?no="+q.value(0).toString()));
}

void Part::insertAuthor() {
  SelectItem *s=new SelectItem(this, "selauthor");
  Q_ASSERT(s);
  s->setData("author/author", "select no, lastname, firstname from author order by lastname");
  if(s->exec()==QDialog::Accepted)
    createPartAuthor(url(), s->url());
}

void Part::addToProject() {
  SelectItem *s=new SelectItem(this, "selproject");
  Q_ASSERT(s);
  s->setData("project/project", "select no, title from project order by title");
  if(s->exec()==QDialog::Accepted) {
    QSqlQuery q("insert into project_bibliography (project_no, tab_no, id_no) values ("+
		no(s->url())+",tabid('part'), "+no(url())+")");
  }
}

void Part::deleteAuthor() {
  deletePartAuthor(url(), partform->authors->url());
}

void Part::createAuthor() {
  QSqlQuery q("select create_author() as no");
  q.first();
  LitUrl u("lit://author/author?no="+q.value(0).toString());
  createPartAuthor(url(), u);
  qApp->postEvent(topLevelWidget(), new UrlEvent(u, false));
}

void Part::showPartPubl() {
  LitUrl u("lit://publication/publication?no="+formcursor.value("publication_no").toString());
  qApp->postEvent(topLevelWidget(), new UrlEvent(u, false));
}


void Part::writebuf(QByteArray buf, int chunkno, QString fileno) {
  
  QFile f("/tmp/writebuf"+QString().setNum(chunkno));
  f.open(IO_WriteOnly);
  f.writeBlock(buf);
  f.close();
  
  QSqlQuery q;
  debug("importing chunk"+QString().setNum(chunkno)+" with partdata_no="+fileno);

  QCString b64=KCodecs::base64Encode(qCompress(buf));

  q.prepare("insert into file (chunk, partdata_no, chunk_no) values ('"+b64+"', "+QCString(fileno)+", "+QCString().setNum(chunkno)+")");
  //  q.bindValue(0, buf);
  q.exec();
}


#define bufsize 1000000

void Part::importFiles() {
  QStringList files = QFileDialog::getOpenFileNames("*",
						    "/home/",
						    this,
						    "Open files dialog"
						    "Select one or more files" );
  QStringList list = files;
  QStringList::Iterator it = list.begin();
  QString mimetype;
  QString pno=no(url());
  
  ImportProgress progress(this, "importprogress");
  progress.show();
  progress.all->setTotalSteps(list.count());
  progress.all->setProgress(0);  

  int i=0; 
  while(it!=list.end()) {
    progress.filename->setText(*it);
    QFile f(*it);
    progress.file->setTotalSteps(f.size()/bufsize+1);
    progress.file->setProgress(0);
    f.open(IO_ReadOnly);
    QByteArray buf=f.readAll();
    
    Convert2Text conv;
    QString text;
    // connect(&conv, SIGNAL(output(QString)), this, SLOT(slotOutput(QString)));
    conv.mime2text(*it, text);

    QSqlQuery q("select create_partdata("+pno+")");
    q.first();
    QString partdata_no=q.value(0).toString();
    debug("partdata_no="+partdata_no);
    q.prepare("update partdata set filename=?, type='"+mimetype+"', astext=? where no="+partdata_no);
    q.bindValue(0, QUrl(*it).fileName());
    q.bindValue(1, text.simplifyWhiteSpace());
    q.exec();
    
    QByteArray chunk(bufsize);
    
    unsigned int j=0;
    while((j+1)*bufsize<buf.size()) {
      memcpy(chunk.data(), buf.data()+j*bufsize, bufsize);
      j++;
      writebuf(chunk, j, partdata_no);
      progress.file->setProgress(j);
      qApp->processEvents();
    }
    chunk.resize(buf.size()-j*bufsize);
    memcpy(chunk.data(), buf.data()+j*bufsize, buf.size()-j*bufsize);
    writebuf(chunk, j+1, partdata_no);
    
    f.close();
 

    progress.all->setProgress(i);
    //    if(progress.all->wasCanceled())
    //      break;
    qApp->processEvents();    
    i++;
    ++it;
  }
  progress.all->setProgress(list.count());
}
