/***************************************************************************
 *                                                                         *
 *   partmemo.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "partmemo.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qcombobox.h>
#include <qsettings.h>
#include "error.h"
#include "editorhelper.h"
#include "global.h"
#include "linklistbox.h"
#include "filllinklistbox.h"
#include "mainwindowimpl.h"
#include "pocketreader.h"
#include <qlayout.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qsimplerichtext.h>
#include <qmime.h> 
#include <qpaintdevicemetrics.h> 
#include <qregexp.h>

PartMemo::PartMemo(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name), toolbar(0L) {    
    partmemoform=new PartMemoForm(this, "partmemoform");
    QHBoxLayout *l=new QHBoxLayout(this);
    l->addWidget(partmemoform);
    
    QSettings settings;
    QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
    if(!backgr.isEmpty()) partmemoform->setErasePixmap(QPixmap(backgr));

    initToolBar();
    new EditorHelper(this, partmemoform->memo);
    setData();
}

void PartMemo::initToolBar() {
  toolbar=new QToolBar(mainWindow());
    
  printAction=new QAction("Print", QIconSet(loadPixmap("print")), "Print", 0, this, "print");
  connect(printAction, SIGNAL(activated()), this, SLOT(print()));


  prAction=new QAction("Retrieve Text from Pocket Reader", QIconSet(loadPixmap("pen.png")), "Retrieve Text from Pocket Reader", 0, this, "pocketreader");
  connect(prAction, SIGNAL(activated()), this, SLOT(pocketReader()));

  printAction->addTo(toolbar);
  prAction->addTo(toolbar);
  toolbar->show();

  printAction->addTo(roMenu());
  prAction->addTo(roMenu());

}

PartMemo::~PartMemo() {
    write();
    delete toolbar;
}

bool PartMemo::setData()
{
    try {
	ReadWriteFormBase::setData();
	LitUrl u=url();
	setWorkingGui(true);
	QSqlQuery query("select publication.title from publication, part where publication.no=part.publication_no and part.no="+no(url()));
	Q_ASSERT(query.first());
	partmemoform->title->setText(query.value(0).toString());

	form.clear();
	formcursor.clear();
	formcursor.setMode(QSqlCursor::Writable);
	formcursor.setName("part");
	if(!formcursor.select(u.query()))
	    throw(Error(formcursor.lastError()));
	if(formcursor.size()!=1) throw Error("Invalid Record!");
	formcursor.first();
	QString no=formcursor.value("no").toString();
	form.setRecord(formcursor.primeUpdate());
	form.insert(partmemoform->chapterno, "partno");	
	form.insert(partmemoform->chapter, "title");
	form.insert(partmemoform->memo, "memo");
	form.insert(partmemoform->modified, "modified");
	form.insert(partmemoform->created, "created");
	form.readFields();
	
	setValid(true);
    } catch(Error &e) {
	setValid(false);
	setReadonlyGui(true);
	QMessageBox::warning(this, "Error", e.message());
	return false;
    }
    setValid(true);
    return true;
}

void PartMemo::setWorkingGui(bool work) {
             ReadWriteFormBase::setInvalidGui(work);
 }

void PartMemo::setReadonlyGui(bool ro) {
    ReadWriteFormBase::setReadonlyGui(ro);
    partmemoform->memo->setEnabled(!ro);    
}

void PartMemo::setInvalidGui(bool inv) {
    ReadWriteFormBase::setInvalidGui(inv);      
    setWorkingGui(false);
    setReadonlyGui(inv);
}

void PartMemo::pocketReader() {
  PocketReader *pr=new PocketReader();
  pr->show();
}

void PartMemo::print() {
  Q_ASSERT(partmemoform->memo);
  QPrinter printer;
  printer.setFullPage( true );
  printer.setDocName("Memo");
  printer.setCreator("Literature");
  
  if(printer.setup(this)) {
    QSqlQuery query("select publication.title, publication.year, part.title, publisher.city, publisher.name from publisher, publication, part where publisher.no=publication.publisher_no and publication.no=part.publication_no and part.no="+no(url()));
    Q_ASSERT(query.first());
    QString publtitle=query.value(0).toString();
    QString year=query.value(1).toString();
    QString parttitle=query.value(2).toString();
    QString city=query.value(3).toString();
    QString publisher=query.value(4).toString();

    QString publauthors;
    query.exec("select lastname, firstname from author, publication_author, part where publication_author.publication_no=part.publication_no and publication_author.author_no=author.no and part.no="+no(url()));
    if(query.first()) {
      publauthors=query.value(0).toString()+", "+query.value(1).toString();;
      if(query.next()) publauthors+=" et al.";
    }
    
    QString partauthors;
    query.exec("select lastname, firstname from author, part_author, part where part_author.author_no=author.no and part_author.part_no="+no(url()));
    if(query.first()) {
      partauthors=query.value(0).toString()+", "+query.value(1).toString();;
      if(query.next()) partauthors+=" et al.";
    }

    QString bib;
    if(!partauthors.isEmpty())
      bib=partauthors+":"+parttitle+".In:"+publauthors+":"+publtitle+"."+
	(city.isEmpty()?QString(""):city+":")+(publisher.isEmpty()?QString(""):publisher+",")+year;
    else
      bib=(!publauthors.isEmpty()?publauthors+":":QString(""))+publtitle+"."+
	(city.isEmpty()?QString(""):city+":")+(publisher.isEmpty()?QString(""):publisher+",")+
	year+","+parttitle;
    qDebug("PartMemo::print(): bib="+bib);

    QPainter p( &printer );
    if ( !p.device() ) return;  	
    QPaintDeviceMetrics metrics( p.device() );
    int dpix = metrics.logicalDpiX();
    int dpiy = metrics.logicalDpiY();
    const double margins = 0.5;
    const double headermargin = 0.2;
    const double pagenumberbox = 0.4;
    const double grace=0.2;

    QRect header=p.boundingRect(int(margins*dpix), int(margins*dpiy),
				int(metrics.width() - 2*margins*dpix),
				int(metrics.height() - 2*margins*dpiy), 
				Qt::AlignLeft | Qt::WordBreak, bib);

    QRect body(header.left(), 
	       int(header.bottom()+headermargin*dpiy), 
	       int(metrics.width() - 2*margins*dpix),
	       int(metrics.height() - header.height()-headermargin*dpiy-pagenumberbox*dpiy-2*margins*dpiy));
    
    QRect pagenumber(int(body.right()-pagenumberbox*dpix), body.bottom(), 
		     int(pagenumberbox*dpix), int(pagenumberbox*dpiy));

    QFont font( "times", 12 );
    QPen pen(black, 3);
    
    Q_ASSERT(partmemoform);
    Q_ASSERT(partmemoform->memo);
    QString text=partmemoform->memo->text();
    
    //    qDebug("PartMemo::print(): text="+text);

    QRect view( body );
    int page = 1;
    do {
      p.setPen(pen);
      p.setFont(font);
      p.drawText(header, Qt::AlignLeft | Qt::WordBreak, bib);
      p.drawLine(header.left(), int(header.bottom()+headermargin*dpiy/2), header.right(), int(header.bottom()+headermargin*dpiy/2));
      
      QString mytext=text;
      int margin=text.length();
      do {
	// qDebug("PartMemo::print(): mytext="+mytext);
	QSimpleRichText richText( mytext, font, partmemoform->memo->context(), partmemoform->memo->styleSheet(), partmemoform->memo->mimeSourceFactory());
 	richText.setWidth(&p, body.width());
	qDebug("PartMemo::print(): richText.height()="+QString().setNum(richText.height()));
	qDebug("PartMemo::print(): body.height()="+QString().setNum(body.height()));	
	qDebug("PartMemo::print(): body.height()-grace*dpiy="+QString().setNum(int(body.height()-grace*dpiy)));
	qDebug("PartMemo::print(): margin="+QString().setNum(margin));

	margin/=2;
	if(richText.height()>body.height()) 
	  mytext=mytext.left(mytext.findRev(QRegExp("\\s|^"), mytext.length()-margin));
	else if(int(richText.height()<body.height()-grace*dpiy) && text.length()>mytext.length()) 
	  mytext+=text.mid(mytext.length(), text.find(QRegExp("\\s|$"), mytext.length()+margin)-mytext.length());
	else break;
      }
      while(true);

      qDebug("PartMemo::print(): text ok, mytext="+mytext);

      text=text.right(text.length()-mytext.length());
      QSimpleRichText richText( mytext, font, partmemoform->memo->context(), partmemoform->memo->styleSheet(), partmemoform->memo->mimeSourceFactory());
      richText.setWidth(&p, body.width());
      richText.draw( &p, body.left(), body.top(), body, colorGroup() );

      p.setFont( font );
      p.drawText(pagenumber, Qt::AlignHCenter|Qt::AlignVCenter, QString::number(page));

      if(text.isEmpty()) break;
      
      printer.newPage();
      page++;

    } while(true);
  }
}
