/***************************************************************************
 *                                                                         *
 *   publisher.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "publisher.h"
#include "liturl.h"
#include <qlayout.h>
#include <qtoolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlcursor.h>
#include <qevent.h>
#include <qsettings.h>
#include "error.h"
#include "global.h"
#include "mainwindowimpl.h"

Publisher::Publisher(LitUrl url, QObject *receiver, QString slot, MainWindow *parent, const char *name): ReadWriteFormBase(url, receiver, slot, parent, name) {    
    publisherform=new PublisherForm(this, "publisherform");
    QHBoxLayout *l=new QHBoxLayout(this);
    l->addWidget(publisherform);
    
    QSettings settings;
    QString backgr = settings.readEntry(QString("/literature/backgr_")+name, "");
    if(!backgr.isEmpty()) publisherform->setErasePixmap(QPixmap(backgr));

    setData();
}


Publisher::~Publisher() {
    write();
}

bool Publisher::setData()
{
    try {
	ReadWriteFormBase::setData();
	LitUrl u=url();
	setWorkingGui(true);
	
	form.clear();
	formcursor.clear();
	formcursor.setMode(QSqlCursor::Writable);
	formcursor.setName("publisher");
	if(!formcursor.select(u.query()))
	    throw(Error(formcursor.lastError()));
	if(formcursor.size()!=1) throw Error("Invalid Record!");
	formcursor.first();
	form.setRecord(formcursor.primeUpdate());
	form.insert(publisherform->pname, "name");	
	form.insert(publisherform->city, "city");
	form.insert(publisherform->modified, "modified");
	form.insert(publisherform->created, "created");
	form.readFields();
	
	setValid(true);
    } catch(Error &e) {
	setValid(false);
	setReadonlyGui(true);
	QMessageBox::warning(this, "Error", e.message());
	return false;
    }
    setValid(true);
    return true;
}

void Publisher::setWorkingGui(bool work) {
             ReadWriteFormBase::setInvalidGui(work);
 }

void Publisher::setReadonlyGui(bool ro) {
    ReadWriteFormBase::setReadonlyGui(ro);
}

void Publisher::setInvalidGui(bool inv) {
    ReadWriteFormBase::setInvalidGui(inv);      
    setWorkingGui(false);
    setReadonlyGui(inv);
}

