/***************************************************************************
 *                                                                         *
 *   readonlyformbase.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "readonlyformbase.h"
#include <qsettings.h>
#include <qaction.h>
#include <qtoolbar.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include "global.h"

void ReadOnlyFormBase::initToolBar() {
  rotoolbar=new QToolBar(mainWindow());
  
  selBackgrAction=new QAction("Select Background", QIconSet(loadPixmap("imagegallery.png")), "Select Background", 0, this, "selbackgr");
  connect(selBackgrAction, SIGNAL(activated()), this, SLOT(selBackgr()));
  selBackgrAction->addTo(rotoolbar);
  
  reloadRecAction=new QAction("Reload Record", QIconSet(loadPixmap("reload.png")), "Reload Record", 0, this, "reloadrec");
  connect(reloadRecAction, SIGNAL(activated()), this, SLOT(reloadRec()));
  reloadRecAction->addTo(rotoolbar);
  
  rotoolbar->show();
  
  reloadRecAction->addTo(roMenu());
  selBackgrAction->addTo(roMenu());
  int c=mainWindow()->menuBar()->count();
  mainWindow()->menuBar()->insertItem("Form", roMenu(), -1, (c>2?c-2:c));
}

QPopupMenu *ReadOnlyFormBase::roMenu() {
  if(!romenu)
    romenu=new QPopupMenu(this);
  ASSERT(romenu);
  return romenu;
}

class Preview : public QLabel, public QFilePreview {
 public:
  Preview( QWidget *parent=0 ) : QLabel( parent ) {}
  void previewUrl( const QUrl &u )
  {
    QString path = u.path();
    QPixmap pix( path );
    if ( pix.isNull() )
      setText( "This is not a pixmap" );
    else
      setPixmap( pix );
  }
};


void ReadOnlyFormBase::selBackgr() {
    Preview* p = new Preview;
    QFileDialog* fd = new QFileDialog( this );
    fd->setContentsPreviewEnabled( TRUE );
    fd->setContentsPreview( p, p );
    fd->setPreviewMode( QFileDialog::Contents );
    QString filename;
    if ( fd->exec() == QDialog::Accepted ) {
        filename = fd->selectedFile();
        QSettings settings;
	settings.setPath("www.test.de", "Literature");
        settings.writeEntry(QString("/literature/backgr_")+name(), filename);
    }
}

void ReadOnlyFormBase::reloadRec() {
    if(QMessageBox::information(this, "Literature", "Do you want to revert the record?",  "&Yes", "&No")!=0) return;
    
    ASSERT(url().isValid());
    setData();
}

QString ReadOnlyFormBase::no(LitUrl o) {
  QSqlQuery q("select no from "+o.table()+" where "+o.query());
  q.first();
  return q.value(0).toString();
}

QString ReadOnlyFormBase::tab(LitUrl o) {
  return "tabid('"+o.table()+"')";
}

QString ReadOnlyFormBase::buildLinkWhereClause(QString target, QString source, QString no) {
  return QString(" ("+target+".no=link.obj_1 and link.tab_1=tabid('"+target+"') and " \
		 " link.obj_2="+no+" and link.tab_2=tabid('"+source+"')) or "\
		 "("+target+".no=link.obj_2 and link.tab_2=tabid('"+target+"') " \
		 " and link.obj_1="+no+" and link.tab_1=tabid('"+source+"'))"); 
}
