/***************************************************************************
 *                                                                         *
 *   readwriteformbase.h								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#ifndef ReadWriteFormBase_h
#define ReadWriteFormBase_h
#include <readonlyformbase.h>
#include <qmainwindow.h>
#include <qevent.h>
#include <qtimer.h>

class QSqlCursor;
class QAction;
class FillLinkListBox;

/**
   @short Base class for read/write dialogs.
   
   Read/write dialogs inherit this class. The class handels locking, readonly/readwrite/invalid 
   gui states, copy/cut/paste, revert/write/delete and multiple threads.   
   
   To be reimplemented by child class:
   @li virtual void setWorkingGui(bool);
   @li virtual void setReadonlyGui(bool);
   @li virtual void setInvalidGui(bool);
   @li virtual bool setData();
   @li Use the variable "form" to manage the fields
*/
class ReadWriteFormBase: public ReadOnlyFormBase {
  Q_OBJECT
    
    private:
  ReadWriteFormBase(const ReadWriteFormBase&);
  ReadWriteFormBase& operator=(const ReadWriteFormBase&);
  bool rec_is_locked;
  bool valid;
  
  /** Disables all gui members of the dialog.
   */
  virtual void lockGui();
  /** Enables all gui members of the dialog.
   */
  virtual void unlockGui();
  /** Receives ThreadFinishedEvent and counts the finished threads.
   */
  void customEvent(QCustomEvent *e);
  /** Locks the current tuple. Returns true (success) or false (already locked) */
  virtual bool lockRec();
  /** Unlocks the current tuple.
   */
  virtual void unlockRec();
  
 protected:	
  QToolBar *rwtoolbar;
  QPtrList<FillLinkListBox> threads;
  QTimer threadtimer;
  /** A virtual method. Loads the data record and fills the form.
      @return false, if an error occured.
  */
  virtual bool setData();
  virtual void initToolBar();
  QAction *writeAction;
  QAction *deleteAction;
  void waitForThreads();
  
 public:
  ReadWriteFormBase(LitUrl u, QObject *receiver, QString slot, QWidget *parent=0, const char *name=0);
  /** Destructor. Unlocks the tuple. 
   */
  ~ReadWriteFormBase();
  bool threadsFinished();
  
  protected slots:
  virtual void setWorkingGui(bool work);
  virtual void setReadonlyGui(bool ro);
  virtual void setInvalidGui(bool inv);
  
  public slots:
  void checkThreads();
  virtual void copy();
  virtual void cut();
  virtual void paste();
  /** Writes the data to the database.
   */
  virtual void write();
  /** Deletes the record.
   */
  virtual void deleteRec();
  void createLink(LitUrl obj1, LitUrl obj2);
  void deleteLink(LitUrl obj1, LitUrl obj2);	
  void createPublicationAuthor(LitUrl publ, LitUrl author);	
  void deletePublicationAuthor(LitUrl publ, LitUrl author);		
  void createPartAuthor(LitUrl part, LitUrl author);		
  void deletePartAuthor(LitUrl part, LitUrl author);
  void createProjectBibliography(LitUrl pr, LitUrl pub);
  void deleteProjectBibliography(LitUrl pr, LitUrl pub);
  void newNote(LitUrl obj);
};

#endif
