/***************************************************************************
 *                                                                         *
 *   search.cpp								   *
 *									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Read the file COPYING for details.					   *
 *									   *
 *   copyright: (C) 2003 by Jan Mueller					   *
 *   email: janmueller7@hotmail.com					   *
 *									   *
 ***************************************************************************/

#include "search.h"
#include "readonlyformbase.h"
#include <qglobal.h>
#include <qlabel.h>
#include <qsqlquery.h>
#include <qlayout.h>
#include <qregexp.h>
#include <qtextbrowser.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qptrlist.h>
#include <threadsearchevent.h>
#include <fillsearchlist.h>
#include "error.h"
#include <qapplication.h>
#include "urlevent.h"
#include "searchtextbrowser.h"

Search::Search(LitUrl u, QObject *receiver, QString slot, QWidget *parent, const char *name): 
  ReadOnlyFormBase(u, receiver, slot, parent, name), fillsearchlist(0L) {
    searchform=new SearchForm(this, "searchform");
    QHBoxLayout *l=new QHBoxLayout(this);
    l->addWidget(searchform);
    Q_ASSERT(searchform);  
    setData();
    connect(searchform->start, SIGNAL(pressed()), this, SLOT(startSearch()));
    connect(searchform->results, SIGNAL(linkClicked(const QString&)), this, SLOT(linkClicked(const QString&)));
    connect(&threadtimer, SIGNAL(timeout()), SLOT(checkThreads()));
  }

void Search::checkThreads() {
  if(fillsearchlist && fillsearchlist->finished()) {
    searchform->start->setEnabled(true);
    threadtimer.stop();
  }
}

void Search::linkClicked(const QString &link) {
  debug("Link clicked: "+link);
  qApp->postEvent(topLevelWidget(), new UrlEvent(link, true));
}
  

void Search::customEvent(QCustomEvent *e) {
  ThreadSearchEvent *l=dynamic_cast<ThreadSearchEvent*>(e);
  if(l) searchform->results->append(l->result());
}

void Search::startSearch() {
  try {
    searchform->start->setEnabled(false);
    QString q=searchform->query->text();
    q=q.simplifyWhiteSpace();
    QStringList sl=QStringList::split(' ', q);

    searchform->results->clear();
    fillsearchlist=new FillSearchList(this, sl);
    fillsearchlist->start();
    threadtimer.start(200, false);
  } catch(Error &e) {
    setValid(false);
    QMessageBox::warning(this, "Error", e.message());
  }
}

bool Search::setData() {
  setValid(true);
  return true;
}

Search::~Search() {
}



